@@ -64,7 +64,7 @@
 
 
 void BOARD_ADAPTER::addText( const EDA_TEXT* aText, CONTAINER_2D_BASE* aContainer,
-                             const BOARD_ITEM* aOwner )
+                             const BOARD_ITEM* aItem )
 {
     KIGFX::GAL_DISPLAY_OPTIONS empty_opts;
     TEXT_ATTRIBUTES            attrs = aText->GetAttributes();
@@ -74,34 +74,25 @@ void BOARD_ADAPTER::addText( const EDA_TEXT* aText, CONTAINER_2D_BASE* aContaine
     if( !font )
         font = KIFONT::FONT::GetFont( wxEmptyString, aText->IsBold(), aText->IsItalic() );
 
-    if( aOwner && aOwner->IsKnockout() )
+    if( aItem && aItem->IsKnockout() )
     {
-        SHAPE_POLY_SET knockouts;
-
-        CALLBACK_GAL callback_gal( empty_opts,
-                // Polygon callback
-                [&]( const SHAPE_LINE_CHAIN& aPoly )
-                {
-                    knockouts.AddOutline( aPoly );
-                } );
-
-        attrs.m_StrokeWidth = aText->GetEffectiveTextPenWidth();
-        attrs.m_Angle = aText->GetDrawRotation();
-
-        callback_gal.SetIsFill( font->IsOutline() );
-        callback_gal.SetIsStroke( font->IsStroke() );
-        callback_gal.SetLineWidth( attrs.m_StrokeWidth );
-        font->Draw( &callback_gal, aText->GetShownText(), aText->GetDrawPos(), attrs );
-
+        int            maxError = m_board->GetDesignSettings().m_MaxError;
         SHAPE_POLY_SET finalPoly;
-        int            margin = attrs.m_StrokeWidth * 1.5 +
-                                    GetKnockoutTextMargin( attrs.m_Size, attrs.m_StrokeWidth );
 
-        aText->TransformBoundingBoxToPolygon( &finalPoly, margin );
-        finalPoly.BooleanSubtract( knockouts, SHAPE_POLY_SET::PM_FAST );
-        finalPoly.Fracture( SHAPE_POLY_SET::PM_FAST );
+        if( aItem->Type() == PCB_FP_TEXT_T )
+        {
+            static_cast<const FP_TEXT*>( aItem )->TransformTextToPolySet( finalPoly, 0, maxError,
+                                                                          ERROR_INSIDE );
+        }
+        else if( aItem->Type() == PCB_TEXT_T )
+        {
+            static_cast<const PCB_TEXT*>( aItem )->TransformTextToPolySet( finalPoly, 0, maxError,
+                                                                           ERROR_INSIDE );
+        }
 
-        ConvertPolygonToTriangles( finalPoly, *aContainer, m_biuTo3Dunits, *aOwner );
+        // Do not call finalPoly.Fracture() here: ConvertPolygonToTriangles() call it
+        // if needed, and Fracture() called twice can create bad results and is useless
+        ConvertPolygonToTriangles( finalPoly, *aContainer, m_biuTo3Dunits, *aItem );
     }
     else
     {
@@ -120,24 +111,24 @@ void BOARD_ADAPTER::addText( const EDA_TEXT* aText, CONTAINER_2D_BASE* aContaine
                     {
                         // Cannot add segments that have the same start and end point
                         aContainer->Add( new FILLED_CIRCLE_2D( pt1_3DU, penWidth_3DU / 2,
-                                                               *aOwner ) );
+                                                               *aItem ) );
                     }
                     else
                     {
                         aContainer->Add( new ROUND_SEGMENT_2D( pt1_3DU, pt2_3DU, penWidth_3DU,
-                                                               *aOwner ) );
+                                                               *aItem ) );
                     }
                 },
                 // Triangulation callback
                 [&]( const VECTOR2I& aPt1, const VECTOR2I& aPt2, const VECTOR2I& aPt3 )
                 {
                     aContainer->Add( new TRIANGLE_2D( TO_SFVEC2F( aPt1 ), TO_SFVEC2F( aPt2 ),
-                                                      TO_SFVEC2F( aPt3 ), *aOwner ) );
+                                                      TO_SFVEC2F( aPt3 ), *aItem ) );
                 } );
 
         attrs.m_Angle = aText->GetDrawRotation();
 
-        font->Draw( &callback_gal, aText->GetShownText(), aText->GetDrawPos(), attrs );
+        font->Draw( &callback_gal, aText->GetShownText( true ), aText->GetDrawPos(), attrs );
     }
 }
 
@@ -210,7 +201,8 @@ void BOARD_ADAPTER::addFootprintShapes( const FOOTPRINT* aFootprint, CONTAINER_2
 
             if( textbox->GetLayer() == aLayerId )
             {
-                addShape( textbox, aContainer, aFootprint );
+                if( textbox->IsBorderEnabled() )
+                    addShape( textbox, aContainer, aFootprint );
                 addText( textbox, aContainer, aFootprint );
             }
 
@@ -269,6 +261,19 @@ void BOARD_ADAPTER::createTrack( const PCB_TRACK* aTrack, CONTAINER_2D_BASE* aDs
     {
         const PCB_ARC* arc = static_cast<const PCB_ARC*>( aTrack );
 
+        if( arc->IsDegenerated() )
+        {
+            // Draw this very small arc like a track segment (a PCB_TRACE_T)
+            PCB_TRACK track( arc->GetParent() );
+            track.SetStart( arc->GetStart() );
+            track.SetEnd( arc->GetEnd() );
+            track.SetWidth( arc->GetWidth() );
+            track.SetLayer( arc->GetLayer() );
+
+            createTrack( &track, aDstContainer );
+            return;
+        }
+
         VECTOR2D  center( arc->GetCenter() );
         EDA_ANGLE arc_angle = arc->GetAngle();
         double    radius = arc->GetRadius();
@@ -634,16 +639,12 @@ void BOARD_ADAPTER::addShape( const PCB_SHAPE* aShape, CONTAINER_2D_BASE* aConta
             if( inner_radius3DU < 0 )
                 inner_radius3DU = 0.0;
 
-            if( outer_radius3DU == 0.0 )
-            {
-                // Don't attempt to render degenerate shapes
-            }
-            else if( aShape->IsFilled() )
+            if( aShape->IsFilled() )
             {
                 aContainer->Add( new FILLED_CIRCLE_2D( center3DU, outer_radius3DU,
                                                        *aOwner ) );
             }
-            else
+            else if( outer_radius3DU > inner_radius3DU )
             {
                 aContainer->Add( new RING_2D( center3DU, inner_radius3DU, outer_radius3DU,
                                               *aOwner ) );
@@ -791,7 +792,7 @@ void BOARD_ADAPTER::buildPadOutlineAsSegments( const PAD* aPad, CONTAINER_2D_BAS
     }
 
     // For other shapes, add outlines as thick segments in polygon buffer
-    const std::shared_ptr<SHAPE_POLY_SET>& corners = aPad->GetEffectivePolygon();
+    const std::shared_ptr<SHAPE_POLY_SET>& corners = aPad->GetEffectivePolygon( ERROR_INSIDE );
     const SHAPE_LINE_CHAIN&                path = corners->COutline( 0 );
 
     for( int j = 0; j < path.PointCount(); j++ )
