@@ -3,7 +3,7 @@
  *lib_tree_model
  * Copyright (C) 2017 Chris Pavlina <pavlina.chris@gmail.com>
  * Copyright (C) 2014 Henner Zeller <h.zeller@acm.org>
- * Copyright (C) 2014-2019 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2014-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -24,14 +24,12 @@
 #include <algorithm>
 #include <eda_pattern_match.h>
 #include <lib_tree_item.h>
-#include <utility>
 #include <pgm_base.h>
 #include <string_utils.h>
 
-// Each node gets this lowest score initially, without any matches applied.
-// Matches will then increase this score depending on match quality.  This way,
-// an empty search string will result in all components being displayed as they
-// have the minimum score. However, in that case, we avoid expanding all the
+// Each node gets this lowest score initially, without any matches applied.  Matches will then
+// increase this score.  This way, an empty search string will result in all components being
+// displayed as they have the minimum score. However, in that case, we avoid expanding all the
 // nodes asd the result is very unspecific.
 static const unsigned kLowestDefaultScore = 1;
 
@@ -92,7 +90,7 @@ void LIB_TREE_NODE::SortNodes()
     std::sort( m_Children.begin(), m_Children.end(),
                []( std::unique_ptr<LIB_TREE_NODE>& a, std::unique_ptr<LIB_TREE_NODE>& b )
                {
-                   return Compare( *a, *b ) > 0;
+                   return Compare( *a, *b );
                } );
 
     for( std::unique_ptr<LIB_TREE_NODE>& node: m_Children )
@@ -100,27 +98,38 @@ void LIB_TREE_NODE::SortNodes()
 }
 
 
-int LIB_TREE_NODE::Compare( LIB_TREE_NODE const& aNode1, LIB_TREE_NODE const& aNode2 )
+bool LIB_TREE_NODE::Compare( LIB_TREE_NODE const& aNode1, LIB_TREE_NODE const& aNode2 )
 {
     if( aNode1.m_Type != aNode2.m_Type )
-        return 0;
+        return aNode1.m_Type < aNode2.m_Type;
 
     // Recently used sorts at top
     if( aNode1.m_Name.StartsWith( wxT( "-- " ) ) )
-        return 1;
+    {
+        if( aNode2.m_Name.StartsWith( wxT( "-- " ) ) )
+        {
+            return aNode1.m_IntrinsicRank > aNode2.m_IntrinsicRank;
+        }
+        else
+        {
+            return true;
+        }
+    }
     else if( aNode2.m_Name.StartsWith( wxT( "-- " ) ) )
-        return 0;
+    {
+        return false;
+    }
 
     // Pinned nodes go next
     if( aNode1.m_Pinned && !aNode2.m_Pinned )
-        return 1;
+        return true;
     else if( aNode2.m_Pinned && !aNode1.m_Pinned )
-        return -1;
+        return false;
 
-    if( aNode1.m_Parent != aNode2.m_Parent )
-        return 0;
+    if( aNode1.m_IntrinsicRank != aNode2.m_IntrinsicRank )
+        return aNode1.m_IntrinsicRank > aNode2.m_IntrinsicRank;
 
-    return aNode1.m_IntrinsicRank - aNode2.m_IntrinsicRank;
+    return reinterpret_cast<const void*>( &aNode1 ) < reinterpret_cast<const void*>( &aNode2 );
 }
 
 
@@ -313,8 +322,6 @@ LIB_TREE_NODE_LIB_ID& LIB_TREE_NODE_LIB::AddItem( LIB_TREE_ITEM* aItem )
 
 void LIB_TREE_NODE_LIB::UpdateScore( EDA_COMBINED_MATCHER& aMatcher, const wxString& aLib )
 {
-    m_Score = 0;
-
     // We need to score leaf nodes, which are usually (but not always) children.
 
     if( m_Children.size() )
