@@ -113,7 +113,7 @@ void PSLIKE_PLOTTER::FlashPadOval( const VECTOR2I& aPadPos, const VECTOR2I& aSiz
     if( aTraceMode == FILLED )
         ThickSegment( a + aPadPos, b + aPadPos, size.x, aTraceMode, nullptr );
     else
-        sketchOval( aPadPos, size, orient, -1 );
+        sketchOval( aPadPos, size, orient, USE_DEFAULT_LINE_WIDTH );
 }
 
 
@@ -967,6 +967,28 @@ void PS_PLOTTER::Text( const VECTOR2I&             aPos,
 }
 
 
+void PS_PLOTTER::PlotText( const VECTOR2I&          aPos,
+                           const COLOR4D&           aColor,
+                           const wxString&          aText,
+                           const TEXT_ATTRIBUTES&   aAttributes,
+                           KIFONT::FONT*            aFont,
+                           void*                    aData )
+{
+    SetCurrentLineWidth( aAttributes.m_StrokeWidth );
+    SetColor( aColor );
+
+    // Draw the hidden postscript text (if requested)
+    if( m_textMode == PLOT_TEXT_MODE::PHANTOM )
+    {
+        std::string ps_test = encodeStringForPlotter( aText );
+        VECTOR2D pos_dev = userToDeviceCoordinates( aPos );
+        fprintf( m_outputFile, "%s %g %g phantomshow\n", ps_test.c_str(), pos_dev.x, pos_dev.y );
+    }
+
+    PLOTTER::PlotText( aPos, aColor, aText, aAttributes, aFont, aData );
+}
+
+
 /**
  * Character widths for Helvetica
  */
