@@ -443,8 +443,8 @@ void SVG_PLOTTER::Circle( const VECTOR2I& pos, int diametre, FILL_T fill, int wi
 }
 
 
-void SVG_PLOTTER::Arc( const VECTOR2I& aCenter, const EDA_ANGLE& aStartAngle,
-                       const EDA_ANGLE& aEndAngle, int aRadius, FILL_T aFill, int aWidth )
+void SVG_PLOTTER::Arc( const VECTOR2D& aCenter, const EDA_ANGLE& aStartAngle,
+                       const EDA_ANGLE& aEndAngle, double aRadius, FILL_T aFill, int aWidth )
 {
     /* Draws an arc of a circle, centered on (xc,yc), with starting point (x1, y1) and ending
      * at (x2, y2). The current pen is used for the outline and the current brush for filling
@@ -633,7 +633,13 @@ void SVG_PLOTTER::PlotImage( const wxImage& aImage, const VECTOR2I& aPos, double
     {
         wxMemoryOutputStream img_stream;
 
-        aImage.SaveFile( img_stream, wxBITMAP_TYPE_PNG );
+        if( m_colorMode )
+            aImage.SaveFile( img_stream, wxBITMAP_TYPE_PNG );
+        else    // Plot in B&W
+        {
+            wxImage image = aImage.ConvertToGreyscale();
+            image.SaveFile( img_stream, wxBITMAP_TYPE_PNG );
+        }
         size_t input_len = img_stream.GetOutputStreamBuffer()->GetBufferSize();
         std::vector<uint8_t> buffer( input_len );
         std::vector<uint8_t> encoded;
@@ -847,3 +853,23 @@ void SVG_PLOTTER::Text( const VECTOR2I&             aPos,
 
     fputs( "</g>", m_outputFile );
 }
+
+
+void SVG_PLOTTER::PlotText( const VECTOR2I& aPos, const COLOR4D& aColor,
+                    const wxString& aText,
+                    const TEXT_ATTRIBUTES& aAttributes,
+                    KIFONT::FONT* aFont,
+                    void* aData )
+{
+    VECTOR2I size = aAttributes.m_Size;
+
+    if( aAttributes.m_Mirrored )
+        size.x = -size.x;
+
+    SVG_PLOTTER::Text( aPos, aColor, aText, aAttributes.m_Angle, size,
+                       aAttributes.m_Halign, aAttributes.m_Valign,
+                       aAttributes.m_StrokeWidth,
+                       aAttributes.m_Italic, aAttributes.m_Bold,
+                       aAttributes.m_Multiline,
+                       aFont, aData );
+}
