@@ -33,6 +33,7 @@
 #include <id.h>
 #include <kiface_base.h>
 #include <kiplatform/app.h>
+#include <lockfile.h>
 #include <pgm_base.h>
 #include <profile.h>
 #include <project/project_file.h>
@@ -69,6 +70,8 @@
 #include <widgets/wx_progress_reporters.h>
 #include <widgets/wx_html_report_box.h>
 
+#include <kiplatform/io.h>
+
 #if wxCHECK_VERSION( 3, 1, 7 )
 #include "widgets/filedlg_hook_save_project.h"
 #else
@@ -100,8 +103,10 @@ bool SCH_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
     {
         msg.Printf( _( "Schematic '%s' is already open." ), wx_filename.GetFullName() );
 
-        if( !OverrideLock( this, msg ) )
+        if( !AskOverrideLock( this, msg ) )
             return false;
+
+        m_file_checker->OverrideLock( false );
     }
 
     if( !AskToSaveChanges() )
@@ -585,6 +590,15 @@ void SCH_EDIT_FRAME::OnImportProject( wxCommandEvent& aEvent )
     if( !AskToSaveChanges() )
         return;
 
+    if( Schematic().RootScreen() && !Schematic().RootScreen()->Items().empty() )
+    {
+        wxString msg = _( "This operation replaces the contents of the current project.\n\n"
+                          "Do you want to proceed?" );
+
+        if( !IsOK( this, msg ) )
+            return;
+    }
+
     // Set the project location if none is set or if we are running in standalone mode
     bool     setProject = Prj().GetProjectFullName().IsEmpty() || Kiface().IsSingle();
     wxString path = wxPathOnly( Prj().GetProjectFullName() );
@@ -633,6 +647,9 @@ void SCH_EDIT_FRAME::OnImportProject( wxCommandEvent& aEvent )
         Schematic().SetProject( nullptr );
         GetSettingsManager()->UnloadProject( &Prj(), false );
 
+        // Clear view before destroying schematic as repaints depend on schematic being valid
+        SetScreen( nullptr );
+
         Schematic().Reset();
 
         wxFileName projectFn( dlg.GetPath() );
@@ -691,6 +708,17 @@ bool SCH_EDIT_FRAME::saveSchematicFile( SCH_SHEET* aSheet, const wxString& aSave
     if( !IsWritable( schematicFileName ) )
         return false;
 
+    wxFileName projectFile( schematicFileName );
+
+    projectFile.SetExt( ProjectFileExtension );
+
+    if( projectFile.FileExists() )
+    {
+        // Save various ERC settings, such as violation severities (which may have been edited
+        // via the ERC dialog as well as the Schematic Setup dialog), ERC exclusions, etc.
+        saveProjectSettings();
+    }
+
     wxString tempFile = wxFileName::CreateTempFileName( wxS( "eeschema" ) );
 
     // Save
@@ -727,6 +755,8 @@ bool SCH_EDIT_FRAME::saveSchematicFile( SCH_SHEET* aSheet, const wxString& aSave
 
     if( success )
     {
+        // Preserve the permissions of the current file
+        KIPLATFORM::IO::DuplicatePermissions( schematicFileName.GetFullPath(), tempFile );
         // Replace the original with the temporary file we just wrote
         success = wxRenameFile( tempFile, schematicFileName.GetFullPath() );
 
@@ -1105,8 +1135,7 @@ bool SCH_EDIT_FRAME::SaveProject( bool aSaveAs )
     }
     else
     {
-        RecordERCExclusions();  // ensure ERC Exclusions list is up to date
-        GetSettingsManager()->SaveProject();
+        saveProjectSettings();
     }
 
     if( !Kiface().IsSingle() )
@@ -1211,10 +1240,13 @@ bool SCH_EDIT_FRAME::importFile( const wxString& aFileName, int aFileType )
         if( !LockFile( aFileName ) )
         {
             wxString msg;
-            msg.Printf( _( "Schematic '%s' is already open." ), filename.GetFullName() );
+            msg.Printf( _( "Schematic '%s' is already open by '%s' at '%s'." ), aFileName,
+                    m_file_checker->GetUsername(), m_file_checker->GetHostname() );
 
-            if( !OverrideLock( this, msg ) )
+            if( !AskOverrideLock( this, msg ) )
                 return false;
+
+            m_file_checker->OverrideLock();
         }
 
         try
