@@ -137,13 +137,12 @@ SCH_SYMBOL::SCH_SYMBOL( const LIB_SYMBOL& aSymbol, const LIB_ID& aLibId,
     m_prefix = UTIL::GetRefDesPrefix( m_part->GetReferenceField().GetText() );
 
     if( aSheet )
-    {
         SetRef( aSheet, UTIL::GetRefDesUnannotated( m_prefix ) );
-    }
 
-    // Inherit the include in bill of materials and board netlist settings from library symbol.
-    m_inBom = aSymbol.GetIncludeInBom();
-    m_onBoard = aSymbol.GetIncludeOnBoard();
+    // Inherit the include in bill of materials and board netlist settings from flattened
+    // library symbol.
+    m_inBom = m_part->GetIncludeInBom();
+    m_onBoard = m_part->GetIncludeOnBoard();
     m_DNP = false;
 
 }
@@ -178,9 +177,6 @@ SCH_SYMBOL::SCH_SYMBOL( const SCH_SYMBOL& aSymbol ) :
     m_onBoard     = aSymbol.m_onBoard;
     m_DNP         = aSymbol.m_DNP;
 
-    if( aSymbol.m_part )
-        SetLibSymbol( new LIB_SYMBOL( *aSymbol.m_part.get() ) );
-
     const_cast<KIID&>( m_Uuid ) = aSymbol.m_Uuid;
 
     m_transform = aSymbol.m_transform;
@@ -192,6 +188,18 @@ SCH_SYMBOL::SCH_SYMBOL( const SCH_SYMBOL& aSymbol ) :
     for( SCH_FIELD& field : m_fields )
         field.SetParent( this );
 
+    m_pins.clear();
+
+    // Copy (and re-parent) the pins
+    for( const std::unique_ptr<SCH_PIN>& pin : aSymbol.m_pins )
+    {
+        m_pins.emplace_back( std::make_unique<SCH_PIN>( *pin ) );
+        m_pins.back()->SetParent( this );
+    }
+
+    if( aSymbol.m_part )
+        SetLibSymbol( new LIB_SYMBOL( *aSymbol.m_part ) );
+
     m_fieldsAutoplaced = aSymbol.m_fieldsAutoplaced;
     m_schLibSymbolName = aSymbol.m_schLibSymbolName;
 }
@@ -334,49 +342,104 @@ wxString SCH_SYMBOL::GetDatasheet() const
 
 void SCH_SYMBOL::UpdatePins()
 {
-    std::map<wxString, wxString> altPinMap;
-    std::map<wxString, KIID>     pinUuidMap;
+    std::map<wxString, wxString>            altPinMap;
+    std::map<wxString, std::set<SCH_PIN*>>  pinUuidMap;
+    std::set<SCH_PIN*>                      unassignedSchPins;
+    std::set<LIB_PIN*>                      unassignedLibPins;
 
     for( const std::unique_ptr<SCH_PIN>& pin : m_pins )
     {
-        pinUuidMap[ pin->GetNumber() ] = pin->m_Uuid;
+        pinUuidMap[ pin->GetNumber() ].insert( pin.get() );
+
+        unassignedSchPins.insert( pin.get() );
 
         if( !pin->GetAlt().IsEmpty() )
             altPinMap[ pin->GetNumber() ] = pin->GetAlt();
     }
 
-    m_pins.clear();
     m_pinMap.clear();
 
     if( !m_part )
         return;
 
-    unsigned i = 0;
-
     std::vector<LIB_PIN*> pins = m_part->GetAllLibPins();
 
     for( LIB_PIN* libPin : pins )
     {
         // NW: Don't filter by unit: this data-structure is used for all instances,
-        // some if which might have different units.
+        // some of which might have different units.
         if( libPin->GetConvert() && m_convert && m_convert != libPin->GetConvert() )
             continue;
 
-        m_pins.push_back( std::make_unique<SCH_PIN>( libPin, this ) );
+        SCH_PIN* pin = nullptr;
 
         auto ii = pinUuidMap.find( libPin->GetNumber() );
 
-        if( ii != pinUuidMap.end() )
-            const_cast<KIID&>( m_pins.back()->m_Uuid ) = ii->second;
+        if( ii == pinUuidMap.end() || ii->second.empty() )
+        {
+            unassignedLibPins.insert( libPin );
+            continue;
+        }
+
+        auto it = ii->second.begin();
+        pin = *it;
+        ii->second.erase( it );
+        pin->SetLibPin( libPin );
+        pin->SetPosition( libPin->GetPosition() );
+
+        unassignedSchPins.erase( pin );
 
         auto iii = altPinMap.find( libPin->GetNumber() );
 
         if( iii != altPinMap.end() )
-            m_pins.back()->SetAlt( iii->second );
+            pin->SetAlt( iii->second );
 
-        m_pinMap[ libPin ] = i;
+        m_pinMap[ libPin ] = pin;
+    }
 
-        ++i;
+    // Add any pins that were not found in the symbol
+    for( LIB_PIN* libPin : unassignedLibPins )
+    {
+        SCH_PIN* pin = nullptr;
+
+        // First try to re-use an existing pin
+        if( !unassignedSchPins.empty() )
+        {
+            auto it = unassignedSchPins.begin();
+            pin = *it;
+            unassignedSchPins.erase( it );
+        }
+        else
+        {
+            // This is a pin that was not found in the symbol, so create a new one.
+            pin = m_pins.emplace_back( std::make_unique<SCH_PIN>( SCH_PIN( libPin, this ) ) ).get();
+        }
+
+        m_pinMap[ libPin ] = pin;
+        pin->SetLibPin( libPin );
+        pin->SetPosition( libPin->GetPosition() );
+        pin->SetNumber( libPin->GetNumber() );
+
+        auto iii = altPinMap.find( libPin->GetNumber() );
+
+        if( iii != altPinMap.end() )
+            pin->SetAlt( iii->second );
+    }
+
+    // If we have any pins left in the symbol that were not found in the library, remove them.
+    for( auto it1 = m_pins.begin(); it1 != m_pins.end() && !unassignedSchPins.empty(); )
+    {
+        auto it2 = unassignedSchPins.find( it1->get() );
+
+        if( it2 != unassignedSchPins.end() )
+        {
+            it1 = m_pins.erase( it1 );
+            unassignedSchPins.erase( it2 );
+        }
+        else
+        {
+            ++it1;
+        }
     }
 }
 
@@ -828,10 +891,11 @@ void SCH_SYMBOL::SetUnitSelection( int aUnitSelection )
 }
 
 
-const wxString SCH_SYMBOL::GetValueFieldText( bool aResolve ) const
+const wxString SCH_SYMBOL::GetValueFieldText( bool aResolve, const SCH_SHEET_PATH* aPath,
+                                              bool aAllowExtraText ) const
 {
     if( aResolve )
-        return GetField( VALUE_FIELD )->GetShownText();
+        return GetField( VALUE_FIELD )->GetShownText( aPath, aAllowExtraText );
 
     return GetField( VALUE_FIELD )->GetText();
 }
@@ -843,10 +907,11 @@ void SCH_SYMBOL::SetValueFieldText( const wxString& aValue )
 }
 
 
-const wxString SCH_SYMBOL::GetFootprintFieldText(  bool aResolve ) const
+const wxString SCH_SYMBOL::GetFootprintFieldText( bool aResolve, const SCH_SHEET_PATH* aPath,
+                                                  bool aAllowExtraText ) const
 {
     if( aResolve )
-        return GetField( FOOTPRINT_FIELD )->GetShownText();
+        return GetField( FOOTPRINT_FIELD )->GetShownText( aPath, aAllowExtraText );
 
     return GetField( FOOTPRINT_FIELD )->GetText();
 }
@@ -900,7 +965,7 @@ void SCH_SYMBOL::GetFields( std::vector<SCH_FIELD*>& aVector, bool aVisibleOnly
     {
         if( aVisibleOnly )
         {
-            if( !field.IsVisible() || field.GetShownText().IsEmpty() )
+            if( !field.IsVisible() || field.GetShownText( nullptr, true ).IsEmpty() )
                 continue;
         }
 
@@ -1056,8 +1121,13 @@ std::vector<LIB_PIN*> SCH_SYMBOL::GetAllLibPins() const
 
 SCH_PIN* SCH_SYMBOL::GetPin( LIB_PIN* aLibPin ) const
 {
-    wxASSERT( m_pinMap.count( aLibPin ) );
-    return m_pins[ m_pinMap.at( aLibPin ) ].get();
+    auto it = m_pinMap.find( aLibPin );
+
+    if( it != m_pinMap.end() )
+        return it->second;
+
+    wxFAIL_MSG_AT( "Pin not found", __FILE__, __LINE__, __FUNCTION__ );
+    return nullptr;
 }
 
 
@@ -1153,10 +1223,14 @@ void SCH_SYMBOL::GetContextualTextVars( wxArrayString* aVars ) const
     aVars->push_back( wxT( "EXCLUDE_FROM_BOM" ) );
     aVars->push_back( wxT( "EXCLUDE_FROM_BOARD" ) );
     aVars->push_back( wxT( "DNP" ) );
+    aVars->push_back( wxT( "SHORT_NET_NAME(<pin_number>)" ) );
+    aVars->push_back( wxT( "NET_NAME(<pin_number>)" ) );
+    aVars->push_back( wxT( "NET_CLASS(<pin_number>)" ) );
+    aVars->push_back( wxT( "PIN_NAME(<pin_number>)" ) );
 }
 
 
-bool SCH_SYMBOL::ResolveTextVar( wxString* token, int aDepth ) const
+bool SCH_SYMBOL::ResolveTextVar( wxString* token, int aDepth, const SCH_SHEET_PATH* aPath ) const
 {
     SCHEMATIC* schematic = Schematic();
 
@@ -1164,6 +1238,8 @@ bool SCH_SYMBOL::ResolveTextVar( wxString* token, int aDepth ) const
     if( !schematic )
         return false;
 
+    SCH_SHEET* sheet = aPath ? aPath->Last() : schematic->CurrentSheet().Last();
+
     if( token->Contains( ':' ) )
     {
         if( schematic->ResolveCrossReference( token, aDepth + 1 ) )
@@ -1176,12 +1252,8 @@ bool SCH_SYMBOL::ResolveTextVar( wxString* token, int aDepth ) const
         {
             if( i == REFERENCE_FIELD )
                 *token = GetRef( &schematic->CurrentSheet(), true );
-            else if( i == VALUE_FIELD )
-                *token = GetValueFieldText( true );
-            else if( i == FOOTPRINT_FIELD )
-                *token = GetFootprintFieldText( true );
             else
-                *token = m_fields[ i ].GetShownText( aDepth + 1 );
+                *token = m_fields[ i ].GetShownText( aPath, false, aDepth + 1 );
 
             return true;
         }
@@ -1192,7 +1264,7 @@ bool SCH_SYMBOL::ResolveTextVar( wxString* token, int aDepth ) const
         if( token->IsSameAs( m_fields[ i ].GetName() )
             || token->IsSameAs( m_fields[ i ].GetName().Upper() ) )
         {
-            *token = m_fields[ i ].GetShownText( aDepth + 1 );
+            *token = m_fields[ i ].GetShownText( aPath, false, aDepth + 1 );
             return true;
         }
     }
@@ -1214,22 +1286,30 @@ bool SCH_SYMBOL::ResolveTextVar( wxString* token, int aDepth ) const
     {
         wxString footprint;
 
-        footprint = GetFootprintFieldText( true );
+        footprint = GetFootprintFieldText( true, aPath, false );
 
         wxArrayString parts = wxSplit( footprint, ':' );
 
-        *token = parts[ 0 ];
+        if( parts.Count() > 0 )
+            *token = parts[ 0 ];
+        else
+            *token = wxEmptyString;
+
         return true;
     }
     else if( token->IsSameAs( wxT( "FOOTPRINT_NAME" ) ) )
     {
         wxString footprint;
 
-        footprint = GetFootprintFieldText( true );
+        footprint = GetFootprintFieldText( true, aPath, false );
 
         wxArrayString parts = wxSplit( footprint, ':' );
 
-        *token = parts[ std::min( 1, (int) parts.size() - 1 ) ];
+        if( parts.Count() > 1 )
+            *token = parts[ std::min( 1, (int) parts.size() - 1 ) ];
+        else
+            *token = wxEmptyString;
+
         return true;
     }
     else if( token->IsSameAs( wxT( "UNIT" ) ) )
@@ -1276,12 +1356,45 @@ bool SCH_SYMBOL::ResolveTextVar( wxString* token, int aDepth ) const
         *token = this->GetDNP() ? _( "DNP" ) : wxT( "" );
         return true;
     }
+    else if( token->StartsWith( wxT( "SHORT_NET_NAME(" ) )
+                 || token->StartsWith( wxT( "NET_NAME(" ) )
+                 || token->StartsWith( wxT( "NET_CLASS(" ) )
+                 || token->StartsWith( wxT( "PIN_NAME(" ) ) )
+    {
+        wxString pinNumber = token->AfterFirst( '(' );
+        pinNumber = pinNumber.BeforeLast( ')' );
+
+        for( SCH_PIN* pin : GetPins( aPath ) )
+        {
+            if( pin->GetNumber() == pinNumber )
+            {
+                if( token->StartsWith( wxT( "PIN_NAME" ) ) )
+                {
+                    *token = pin->GetAlt().IsEmpty() ? pin->GetName() : pin->GetAlt();
+                    return true;
+                }
+
+                SCH_CONNECTION* conn = pin->Connection( aPath );
+
+                if( !conn )
+                    *token = wxEmptyString;
+                else if( token->StartsWith( wxT( "SHORT_NET_NAME" ) ) )
+                    *token = conn->LocalName();
+                else if( token->StartsWith( wxT( "NET_NAME" ) ) )
+                    *token = conn->Name();
+                else if( token->StartsWith( wxT( "NET_CLASS" ) ) )
+                    *token = pin->GetEffectiveNetClass( aPath )->GetName();
+
+                return true;
+            }
+        }
+    }
 
     // See if parent can resolve it (this will recurse to ancestors)
 
-    if( SCH_SHEET* sheet = schematic->CurrentSheet().Last() )
+    if( sheet )
     {
-        if( sheet->ResolveTextVar( token, aDepth + 1 ) )
+        if( sheet->ResolveTextVar( aPath, token, aDepth + 1 ) )
             return true;
     }
 
@@ -1622,7 +1735,17 @@ BOX2I SCH_SYMBOL::doGetBoundingBox( bool aIncludePins, bool aIncludeFields ) con
 
 BOX2I SCH_SYMBOL::GetBodyBoundingBox() const
 {
-    return doGetBoundingBox( false, false );
+    try
+    {
+        return doGetBoundingBox( false, false );
+    }
+    catch( const boost::bad_pointer& exc )
+    {
+        // This may be overkill and could be an assertion but we are more likely to
+        // find any boost pointer container errors this way.
+        wxLogError( wxT( "Boost bad pointer exception '%s' occurred." ), exc.what() );
+        return BOX2I();
+    }
 }
 
 
@@ -1652,13 +1775,20 @@ void SCH_SYMBOL::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_
         {
             if( m_part->IsPower() )
             {
-                aList.emplace_back( _( "Power symbol" ), GetValueFieldText( true ) );
+                // Don't use GetShownText(); we want to see the variable references here
+                aList.emplace_back( _( "Power symbol" ),
+                                    KIUI::EllipsizeStatusText( aFrame, GetField( VALUE_FIELD )->GetText() ) );
             }
             else
             {
-                aList.emplace_back( _( "Reference" ), GetRef( currentSheet ) );
-                aList.emplace_back( _( "Value" ), GetValueFieldText( true ) );
-                aList.emplace_back( _( "Name" ), UnescapeString( GetLibId().GetLibItemName() ) );
+                aList.emplace_back( _( "Reference" ),
+                                    UnescapeString( GetRef( currentSheet ) ) );
+
+                // Don't use GetShownText(); we want to see the variable references here
+                aList.emplace_back( _( "Value" ),
+                                    KIUI::EllipsizeStatusText( aFrame, GetField( VALUE_FIELD )->GetText() ) );
+                aList.emplace_back( _( "Name" ),
+                                    KIUI::EllipsizeStatusText( aFrame, GetLibId().GetLibItemName() ) );
             }
 
 #if 0       // Display symbol flags, for debug only
@@ -1686,7 +1816,8 @@ void SCH_SYMBOL::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_
             }
 
             // Display the current associated footprint, if exists.
-            msg = GetFootprintFieldText( true );
+            // Don't use GetShownText(); we want to see the variable references here
+            msg = KIUI::EllipsizeStatusText( aFrame, GetField( FOOTPRINT_FIELD )->GetText() );
 
             if( msg.IsEmpty() )
                 msg = _( "<Unknown>" );
@@ -1701,9 +1832,11 @@ void SCH_SYMBOL::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_
     else
     {
         aList.emplace_back( _( "Reference" ), GetRef( currentSheet ) );
-
-        aList.emplace_back( _( "Value" ), GetValueFieldText( true ) );
-        aList.emplace_back( _( "Name" ), GetLibId().GetLibItemName() );
+        // Don't use GetShownText(); we want to see the variable references here
+        aList.emplace_back( _( "Value" ),
+                            KIUI::EllipsizeStatusText( aFrame, GetField( VALUE_FIELD )->GetText() ) );
+        aList.emplace_back( _( "Name" ),
+                            KIUI::EllipsizeStatusText( aFrame, GetLibId().GetLibItemName() ) );
 
         wxString libNickname = GetLibId().GetLibNickname();
 
@@ -1900,8 +2033,8 @@ LIB_ITEM* SCH_SYMBOL::GetDrawItem( const VECTOR2I& aPosition, KICAD_T aType )
 wxString SCH_SYMBOL::GetItemDescription( UNITS_PROVIDER* aUnitsProvider ) const
 {
     return wxString::Format( _( "Symbol %s [%s]" ),
-                             GetField( REFERENCE_FIELD )->GetShownText(),
-                             UnescapeString( GetLibId().GetLibItemName() ) );
+                             KIUI::EllipsizeMenuText( GetField( REFERENCE_FIELD )->GetText() ),
+                             KIUI::EllipsizeMenuText( GetLibId().GetLibItemName() ) );
 }
 
 
@@ -2148,6 +2281,17 @@ void SCH_SYMBOL::Plot( PLOTTER* aPlotter, bool aBackground ) const
                 tempPin->SetFlags( IS_DANGLING );
         }
 
+        for( LIB_ITEM& item : tempSymbol.GetDrawItems() )
+        {
+            if( EDA_TEXT* text = dynamic_cast<EDA_TEXT*>( &item ) )
+            {
+                // Use SCH_FIELD's text resolver
+                SCH_FIELD dummy( (SCH_ITEM*) this, -1 );
+                dummy.SetText( text->GetText() );
+                text->SetText( dummy.GetShownText( false ) );
+            }
+        }
+
         TRANSFORM temp = GetTransform();
         aPlotter->StartBlock( nullptr );
 
@@ -2156,8 +2300,11 @@ void SCH_SYMBOL::Plot( PLOTTER* aPlotter, bool aBackground ) const
             tempSymbol.Plot( aPlotter, GetUnit(), GetConvert(), local_background, m_pos, temp,
                              GetDNP() );
 
-            for( const SCH_FIELD& field : m_fields )
+            for( SCH_FIELD field : m_fields )
+            {
+                field.ClearRenderCache();
                 field.Plot( aPlotter, local_background );
+            }
         }
 
         if( m_DNP )
@@ -2165,12 +2312,13 @@ void SCH_SYMBOL::Plot( PLOTTER* aPlotter, bool aBackground ) const
 
         // Plot attributes to a hypertext menu
         std::vector<wxString> properties;
+        SCH_SHEET_PATH*       sheet = &Schematic()->CurrentSheet();
 
         for( const SCH_FIELD& field : GetFields() )
         {
             properties.emplace_back( wxString::Format( wxT( "!%s = %s" ),
                                                        field.GetName(),
-                                                       field.GetShownText() ) );
+                                                       field.GetShownText( sheet, false) ) );
         }
 
         properties.emplace_back( wxString::Format( wxT( "!%s = %s" ),
@@ -2188,7 +2336,7 @@ void SCH_SYMBOL::Plot( PLOTTER* aPlotter, bool aBackground ) const
         aPlotter->EndBlock( nullptr );
 
         if( !m_part->IsPower() )
-            aPlotter->Bookmark( GetBoundingBox(), GetField( REFERENCE_FIELD )->GetShownText(), _("Symbols") );
+            aPlotter->Bookmark( GetBoundingBox(), GetRef( sheet ), _( "Symbols" ) );
     }
 }
 
@@ -2298,3 +2446,10 @@ bool SCH_SYMBOL::IsSymbolLikePowerGlobalLabel() const
     return pin_list[0]->GetType() == ELECTRICAL_PINTYPE::PT_POWER_IN;
 }
 
+
+bool SCH_SYMBOL::IsPower() const
+{
+    wxCHECK( m_part, false );
+
+    return m_part->IsPower();
+}
