@@ -327,7 +327,8 @@ public:
      *
      * @param aDepth a counter to limit recursion and circular references.
      */
-    bool ResolveTextVar( wxString* token, int aDepth = 0 ) const;
+    bool ResolveTextVar( wxString* token, int aDepth = 0,
+                         const SCH_SHEET_PATH* aPath = nullptr ) const;
 
     void GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITEM>& aList ) override;
 
@@ -461,10 +462,12 @@ public:
         m_fields = aFields;     // vector copying, length is changed possibly
     }
 
-    const wxString GetValueFieldText( bool aResolve ) const;
+    const wxString GetValueFieldText( bool aResolve, const SCH_SHEET_PATH* aPath,
+                                      bool aAllowExtraText ) const;
     void SetValueFieldText( const wxString& aValue );
 
-    const wxString GetFootprintFieldText( bool aResolve ) const;
+    const wxString GetFootprintFieldText( bool aResolve, const SCH_SHEET_PATH* aPath,
+                                          bool aAllowExtraText ) const;
     void SetFootprintFieldText( const wxString& aFootprint );
 
     /**
@@ -762,6 +765,8 @@ public:
      */
     bool IsSymbolLikePowerGlobalLabel() const;
 
+    bool IsPower() const;
+
 private:
     BOX2I doGetBoundingBox( bool aIncludePins, bool aIncludeFields ) const;
 
@@ -795,7 +800,7 @@ private:
     std::unique_ptr< LIB_SYMBOL >          m_part;      ///< a flattened copy of the LIB_SYMBOL
                                                         ///<   from the PROJECT's libraries.
     std::vector<std::unique_ptr<SCH_PIN>>  m_pins;      ///< a SCH_PIN for every LIB_PIN (all units)
-    std::unordered_map<LIB_PIN*, unsigned> m_pinMap;    ///< library pin pointer : SCH_PIN's index
+    std::unordered_map<LIB_PIN*, SCH_PIN*> m_pinMap;    ///< library pin pointer : SCH_PIN's index
 
     bool        m_isInNetlist;  ///< True if the symbol should appear in the netlist
     bool        m_inBom;        ///< True to include in bill of materials export.
