@@ -149,8 +149,6 @@ int SCH_DRAWING_TOOLS::PlaceSymbol( const TOOL_EVENT& aEvent )
     auto addSymbol =
             [&]( SCH_SYMBOL* aSymbol )
             {
-                m_frame->SaveCopyForRepeatItem( aSymbol );
-
                 m_toolMgr->RunAction( EE_ACTIONS::clearSelection, true );
                 m_selectionTool->AddItemToSel( aSymbol );
 
@@ -249,7 +247,7 @@ int SCH_DRAWING_TOOLS::PlaceSymbol( const TOOL_EVENT& aEvent )
         bool isSyntheticClick = symbol && evt->IsActivate() && evt->HasPosition()
                                 && evt->Matches( aEvent );
 
-        if( evt->IsCancelInteractive() )
+        if( evt->IsCancelInteractive() || evt->IsAction( &ACTIONS::undo ) )
         {
             m_frame->GetInfoBar()->Dismiss();
 
@@ -343,6 +341,8 @@ int SCH_DRAWING_TOOLS::PlaceSymbol( const TOOL_EVENT& aEvent )
                 if( m_frame->eeconfig()->m_AutoplaceFields.enable )
                     symbol->AutoplaceFields( /* aScreen */ nullptr, /* aManual */ false );
 
+                m_frame->SaveCopyForRepeatItem( symbol );
+
                 symbol->ClearEditFlags();
                 m_view->Update( symbol );
                 m_frame->GetScreen()->Update( symbol );
@@ -443,6 +443,10 @@ int SCH_DRAWING_TOOLS::PlaceSymbol( const TOOL_EVENT& aEvent )
         {
             cleanup();
         }
+        else if( evt->IsAction( &ACTIONS::redo ) )
+        {
+            wxBell();
+        }
         else
         {
             evt->SetPassEvent();
@@ -538,7 +542,7 @@ int SCH_DRAWING_TOOLS::PlaceImage( const TOOL_EVENT& aEvent )
         bool isSyntheticClick = image && evt->IsActivate() && evt->HasPosition()
                                 && evt->Matches( aEvent );
 
-        if( evt->IsCancelInteractive() )
+        if( evt->IsCancelInteractive() || evt->IsAction( &ACTIONS::undo ) )
         {
             m_frame->GetInfoBar()->Dismiss();
 
@@ -674,6 +678,10 @@ int SCH_DRAWING_TOOLS::PlaceImage( const TOOL_EVENT& aEvent )
         {
             cleanup();
         }
+        else if( evt->IsAction( &ACTIONS::redo ) )
+        {
+            wxBell();
+        }
         else
         {
             evt->SetPassEvent();
@@ -783,7 +791,7 @@ int SCH_DRAWING_TOOLS::SingleClickPlace( const TOOL_EVENT& aEvent )
         cursorPos = grid.BestSnapAnchor( cursorPos, LAYER_CONNECTABLE, nullptr );
         controls->ForceCursorPosition( true, cursorPos );
 
-        if( evt->IsCancelInteractive() )
+        if( evt->IsCancelInteractive() || evt->IsAction( &ACTIONS::undo ) )
         {
             m_frame->PopTool( aEvent );
             break;
@@ -925,6 +933,10 @@ int SCH_DRAWING_TOOLS::SingleClickPlace( const TOOL_EVENT& aEvent )
                 evt->SetPassEvent();
             }
         }
+        else if( evt->IsAction( &ACTIONS::redo ) )
+        {
+            wxBell();
+        }
         else
         {
             evt->SetPassEvent();
@@ -1239,7 +1251,7 @@ int SCH_DRAWING_TOOLS::TwoClickPlace( const TOOL_EVENT& aEvent )
                     }
                 };
 
-        if( evt->IsCancelInteractive() )
+        if( evt->IsCancelInteractive() || evt->IsAction( &ACTIONS::undo ) )
         {
             m_frame->GetInfoBar()->Dismiss();
 
@@ -1478,6 +1490,10 @@ int SCH_DRAWING_TOOLS::TwoClickPlace( const TOOL_EVENT& aEvent )
         {
             cleanup();
         }
+        else if( evt->IsAction( &ACTIONS::redo ) )
+        {
+            wxBell();
+        }
         else
         {
             evt->SetPassEvent();
@@ -1554,7 +1570,7 @@ int SCH_DRAWING_TOOLS::DrawShape( const TOOL_EVENT& aEvent )
         bool isSyntheticClick = item && evt->IsActivate() && evt->HasPosition()
                                 && evt->Matches( aEvent );
 
-        if( evt->IsCancelInteractive() )
+        if( evt->IsCancelInteractive() || evt->IsAction( &ACTIONS::undo ) )
         {
             if( item )
             {
@@ -1698,6 +1714,10 @@ int SCH_DRAWING_TOOLS::DrawShape( const TOOL_EVENT& aEvent )
 
             m_menu.ShowContextMenu( m_selectionTool->GetSelection() );
         }
+        else if( evt->IsAction( &ACTIONS::redo ) )
+        {
+            wxBell();
+        }
         else
         {
             evt->SetPassEvent();
@@ -1765,7 +1785,7 @@ int SCH_DRAWING_TOOLS::DrawSheet( const TOOL_EVENT& aEvent )
         bool isSyntheticClick = sheet && evt->IsActivate() && evt->HasPosition()
                                 && evt->Matches( aEvent );
 
-        if( evt->IsCancelInteractive() )
+        if( evt->IsCancelInteractive() || evt->IsAction( &ACTIONS::undo ) )
         {
             m_frame->GetInfoBar()->Dismiss();
 
@@ -1886,6 +1906,10 @@ int SCH_DRAWING_TOOLS::DrawSheet( const TOOL_EVENT& aEvent )
 
             m_menu.ShowContextMenu( m_selectionTool->GetSelection() );
         }
+        else if( evt->IsAction( &ACTIONS::redo ) )
+        {
+            wxBell();
+        }
         else
         {
             evt->SetPassEvent();
