@@ -924,6 +924,9 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
         for( EDA_ITEM* item : selection )
             updateItem( item, true );
 
+        if( selection.GetSize() == 1 && selection.Front()->IsNew() )
+            m_frame->SaveCopyForRepeatItem( static_cast<SCH_ITEM*>( selection.Front() ) );
+
         m_selectionTool->RemoveItemsFromSel( &m_dragAdditions, QUIET_MODE );
 
         // If we move items away from a junction, we _may_ want to add a junction there
@@ -1165,7 +1168,11 @@ void SCH_MOVE_TOOL::getConnectedDragItems( SCH_ITEM* aSelectedItem, const VECTOR
 
                 newWire->SetFlags( IS_NEW );
                 newWire->SetConnectivityDirty( true );
-                newWire->SetLastResolvedState( selected );
+
+                if( dynamic_cast<const SCH_LINE*>( selected ) )
+                    newWire->SetLastResolvedState( selected );
+                else if( dynamic_cast<const SCH_LINE*>( fixed ) )
+                    newWire->SetLastResolvedState( fixed );
 
                 newWire->SetEndPoint( end );
                 m_frame->AddToScreen( newWire, m_frame->GetScreen() );
