@@ -428,8 +428,9 @@ const bool PLUGIN_CONTENT_MANAGER::CacheRepository( const wxString& aRepositoryI
 
     wxString url = std::get<2>( *repository_tuple );
 
-    nlohmann::json js;
-    PCM_REPOSITORY current_repo;
+    nlohmann::json  js;
+    PCM_REPOSITORY  current_repo;
+    PCM_REPOSITORY& current_repo_ref = current_repo;
 
     std::shared_ptr<PROGRESS_REPORTER> reporter;
 
@@ -451,9 +452,18 @@ const bool PLUGIN_CONTENT_MANAGER::CacheRepository( const wxString& aRepositoryI
 
     if( repo_cache.FileExists() && packages_cache.FileExists() )
     {
-        std::ifstream repo_stream( repo_cache.GetFullPath().ToUTF8() );
-        repo_stream >> js;
-        PCM_REPOSITORY saved_repo = js.get<PCM_REPOSITORY>();
+        std::ifstream  repo_stream( repo_cache.GetFullPath().ToUTF8() );
+        PCM_REPOSITORY saved_repo;
+        try
+        {
+            repo_stream >> js;
+            saved_repo = js.get<PCM_REPOSITORY>();
+        }
+        catch( ... )
+        {
+            if( m_dialog )
+                wxLogError( _( "Failed to parse locally stored repository.json." ) );
+        }
 
         if( saved_repo.packages.update_timestamp == current_repo.packages.update_timestamp )
         {
@@ -513,12 +523,13 @@ const bool PLUGIN_CONTENT_MANAGER::CacheRepository( const wxString& aRepositoryI
         packages_cache_stream << std::setw( 4 ) << js << std::endl;
 
         m_repository_cache[aRepositoryId] = std::move( current_repo );
+        current_repo_ref = m_repository_cache[aRepositoryId];
     }
 
-    if( current_repo.resources )
+    if( current_repo_ref.resources )
     {
         // Check resources file date, redownload if needed
-        PCM_RESOURCE_REFERENCE& resources = *current_repo.resources;
+        PCM_RESOURCE_REFERENCE& resources = *current_repo_ref.resources;
 
         wxFileName resource_file( repo_cache.GetPath(), wxT( "resources.zip" ) );
 
