@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2022 Mark Roszko <mark.roszko@gmail.com>
  * Copyright (C) 2016 Cirilo Bernardo <cirilo.bernardo@gmail.com>
- * Copyright (C) 2016-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2016-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -38,6 +38,7 @@
 
 #include <footprint.h>
 #include <pad.h>
+#include <kiplatform/io.h>
 
 #include "step_pcb_model.h"
 #include "streamwrapper.h"
@@ -93,19 +94,9 @@
 static constexpr double USER_PREC = 1e-4;
 static constexpr double USER_ANGLE_PREC = 1e-6;
 
-// minimum PCB thickness in mm (2 microns assumes a very thin polyimide film)
-static constexpr double THICKNESS_MIN = 0.002;
-
-// default PCB thickness in mm
-static constexpr double THICKNESS_DEFAULT = 1.6;
-
 // nominal offset from the board
 static constexpr double BOARD_OFFSET = 0.05;
 
-// min. length**2 below which 2 points are considered coincident
-static constexpr double MIN_LENGTH2 = STEPEXPORT_MIN_DISTANCE * STEPEXPORT_MIN_DISTANCE;
-
-
 // supported file types
 enum FormatType
 {
@@ -194,12 +185,12 @@ STEP_PCB_MODEL::STEP_PCB_MODEL( const wxString& aPcbName )
     m_components = 0;
     m_precision = USER_PREC;
     m_angleprec = USER_ANGLE_PREC;
-    m_thickness = THICKNESS_DEFAULT;
-    m_minDistance2 = MIN_LENGTH2;
+    m_boardThickness = BOARD_THICKNESS_DEFAULT_MM;
+    m_mergeOCCMaxDist = OCC_MAX_DISTANCE_TO_MERGE_POINTS;
     m_minx = 1.0e10;    // absurdly large number; any valid PCB X value will be smaller
     m_pcbName = aPcbName;
-    BRepBuilderAPI::Precision( STEPEXPORT_MIN_DISTANCE );
-    m_maxError = 5000;      // 5 microns
+    BRepBuilderAPI::Precision( m_mergeOCCMaxDist );
+    m_maxError = pcbIUScale.mmToIU( ARC_TO_SEGMENT_MAX_ERROR_MM );
 }
 
 
@@ -214,16 +205,21 @@ bool STEP_PCB_MODEL::AddPadHole( const PAD* aPad, const VECTOR2D& aOrigin )
     if( NULL == aPad || !aPad->GetDrillSize().x )
         return false;
 
-    VECTOR2I pos = aPad->GetPosition();
+    VECTOR2I     pos = aPad->GetPosition();
+    const double margin = 0.01; // a small margin on the Z axix to be sure the hole
+                                // is bigger than the board with copper
+                                // must be > OCC_MAX_DISTANCE_TO_MERGE_POINTS
+    double holeZsize = m_boardThickness + ( margin * 2 );
 
     if( aPad->GetDrillShape() == PAD_DRILL_SHAPE_CIRCLE )
     {
         TopoDS_Shape s =
-                BRepPrimAPI_MakeCylinder( pcbIUScale.IUTomm( aPad->GetDrillSize().x ) * 0.5, m_thickness * 2.0 ).Shape();
+                BRepPrimAPI_MakeCylinder( pcbIUScale.IUTomm( aPad->GetDrillSize().x ) * 0.5, holeZsize )
+                        .Shape();
         gp_Trsf shift;
         shift.SetTranslation( gp_Vec( pcbIUScale.IUTomm( pos.x - aOrigin.x ),
-                                          -pcbIUScale.IUTomm( pos.y - aOrigin.y ),
-                                          -m_thickness * 0.5 ) );
+                                      -pcbIUScale.IUTomm( pos.y - aOrigin.y ),
+                                      -margin ) );
         BRepBuilderAPI_Transform hole( s, shift );
         m_cutouts.push_back( hole.Shape() );
         return true;
@@ -240,7 +236,7 @@ bool STEP_PCB_MODEL::AddPadHole( const PAD* aPad, const VECTOR2D& aOrigin )
 
     if( holeOutlines.OutlineCount() > 0 )
     {
-        if( MakeShape( hole, holeOutlines.COutline( 0 ), m_thickness, aOrigin ) )
+        if( MakeShape( hole, holeOutlines.COutline( 0 ), holeZsize, 0.0, aOrigin ) )
         {
             m_cutouts.push_back( hole );
         }
@@ -312,11 +308,11 @@ bool STEP_PCB_MODEL::AddComponent( const std::string& aFileNameUTF8, const std::
 void STEP_PCB_MODEL::SetPCBThickness( double aThickness )
 {
     if( aThickness < 0.0 )
-        m_thickness = THICKNESS_DEFAULT;
-    else if( aThickness < THICKNESS_MIN )
-        m_thickness = THICKNESS_MIN;
+        m_boardThickness = BOARD_THICKNESS_DEFAULT_MM;
+    else if( aThickness < BOARD_THICKNESS_MIN_MM )
+        m_boardThickness = BOARD_THICKNESS_MIN_MM;
     else
-        m_thickness = aThickness;
+        m_boardThickness = aThickness;
 }
 
 
@@ -328,24 +324,42 @@ void STEP_PCB_MODEL::SetBoardColor( double r, double g, double b )
 }
 
 
-void STEP_PCB_MODEL::SetMinDistance( double aDistance )
+void STEP_PCB_MODEL::OCCSetMergeMaxDistance( double aDistance )
 {
     // Ensure a minimal value (in mm)
-    aDistance = std::max( aDistance, STEPEXPORT_MIN_ACCEPTABLE_DISTANCE );
-
-    // m_minDistance2 keeps a squared distance value
-    m_minDistance2 = aDistance * aDistance;
-    BRepBuilderAPI::Precision( aDistance );
+    m_mergeOCCMaxDist = aDistance;
+    BRepBuilderAPI::Precision( m_mergeOCCMaxDist );
 }
 
+
 bool STEP_PCB_MODEL::isBoardOutlineValid()
 {
     return m_pcb_labels.size() > 0;
 }
 
 
-bool STEP_PCB_MODEL::MakeShape( TopoDS_Shape& aShape, const SHAPE_LINE_CHAIN& aChain,
-                                double aThickness, const VECTOR2D& aOrigin )
+// A helper function to know if a SHAPE_LINE_CHAIN is encoding a circle
+static bool IsChainCircle( const SHAPE_LINE_CHAIN& aChain )
+{
+    // If aChain is a circle it
+    // - contains only one arc
+    // - this arc has the same start and end point
+    const std::vector<SHAPE_ARC>& arcs = aChain.CArcs();
+
+    if( arcs.size() == 1 )
+    {
+        const SHAPE_ARC& arc = arcs[0];
+
+        if( arc. GetP0() == arc.GetP1() )
+            return true;
+    }
+
+    return false;
+}
+
+bool STEP_PCB_MODEL::MakeShapeAsCylinder( TopoDS_Shape& aShape,
+                                          const SHAPE_LINE_CHAIN& aChain, double aThickness,
+                                          double aZposition, const VECTOR2D& aOrigin )
 {
     if( !aShape.IsNull() )
         return false; // there is already data in the shape object
@@ -353,12 +367,47 @@ bool STEP_PCB_MODEL::MakeShape( TopoDS_Shape& aShape, const SHAPE_LINE_CHAIN& aC
     if( !aChain.IsClosed() )
         return false; // the loop is not closed
 
+    const std::vector<SHAPE_ARC>& arcs = aChain.CArcs();
+    const SHAPE_ARC& arc = arcs[0];
+
+    TopoDS_Shape base_shape;
+    base_shape = BRepPrimAPI_MakeCylinder(
+                        pcbIUScale.IUTomm( arc.GetRadius() ), aThickness ).Shape();
+    gp_Trsf shift;
+    shift.SetTranslation( gp_Vec( pcbIUScale.IUTomm( arc.GetCenter().x - aOrigin.x ),
+                                  -pcbIUScale.IUTomm( arc.GetCenter().y - aOrigin.y ),
+                                  aZposition ) );
+    BRepBuilderAPI_Transform round_shape( base_shape, shift );
+    aShape = round_shape;
+
+    if( aShape.IsNull() )
+    {
+        ReportMessage( wxT( "failed to create a cylinder vertical shape\n" ) );
+        return false;
+    }
+
+    return true;
+}
+
+bool STEP_PCB_MODEL::MakeShape( TopoDS_Shape& aShape, const SHAPE_LINE_CHAIN& aChain,
+                                double aThickness, double aZposition, const VECTOR2D& aOrigin )
+{
+    if( !aShape.IsNull() )
+        return false; // there is already data in the shape object
+
+    if( !aChain.IsClosed() )
+        return false; // the loop is not closed
+
+    // a SHAPE_LINE_CHAIN that is in fact a circle (one 360deg arc) is exported as cylinder
+    if( IsChainCircle( aChain ) )
+        return MakeShapeAsCylinder( aShape, aChain, aThickness, aZposition, aOrigin );
+
     BRepBuilderAPI_MakeWire wire;
     TopoDS_Edge             edge;
     bool                    success = true;
 
     gp_Pnt start = gp_Pnt( pcbIUScale.IUTomm( aChain.CPoint( 0 ).x - aOrigin.x ),
-                           -pcbIUScale.IUTomm( aChain.CPoint( 0 ).y - aOrigin.y ), 0.0 );
+                           -pcbIUScale.IUTomm( aChain.CPoint( 0 ).y - aOrigin.y ), aZposition );
 
     for( int j = 0; j < aChain.PointCount(); j++ )
     {
@@ -369,19 +418,18 @@ bool STEP_PCB_MODEL::MakeShape( TopoDS_Shape& aShape, const SHAPE_LINE_CHAIN& aC
         if( next >= aChain.PointCount() )
         {
             end = gp_Pnt( pcbIUScale.IUTomm( aChain.CPoint( 0 ).x - aOrigin.x ),
-                          -pcbIUScale.IUTomm( aChain.CPoint( 0 ).y - aOrigin.y ), 0.0 );
+                          -pcbIUScale.IUTomm( aChain.CPoint( 0 ).y - aOrigin.y ), aZposition );
         }
         else
         {
             end = gp_Pnt( pcbIUScale.IUTomm( aChain.CPoint( next ).x - aOrigin.x ),
-                          -pcbIUScale.IUTomm( aChain.CPoint( next ).y - aOrigin.y ), 0.0 );
+                          -pcbIUScale.IUTomm( aChain.CPoint( next ).y - aOrigin.y ), aZposition );
         }
 
         // Do not export very small segments: they can create broken outlines
         double seg_len = std::abs( end.X() - start.X()) + std::abs(start.Y() - end.Y() );
-        double min_len = 0.0001;    // In mm, i.e. 0.1 micron
 
-        if( seg_len < min_len )
+        if( seg_len <= m_mergeOCCMaxDist )
             continue;
 
         try
@@ -460,7 +508,7 @@ bool STEP_PCB_MODEL::CreatePCB( SHAPE_POLY_SET& aOutline, VECTOR2D aOrigin )
 
         TopoDS_Shape curr_brd;
 
-        if( !MakeShape( curr_brd, outline, m_thickness, aOrigin ) )
+        if( !MakeShape( curr_brd, outline, m_boardThickness, 0.0, aOrigin ) )
         {
             // Error
             ReportMessage( wxString::Format(
@@ -482,7 +530,7 @@ bool STEP_PCB_MODEL::CreatePCB( SHAPE_POLY_SET& aOutline, VECTOR2D aOrigin )
             const SHAPE_LINE_CHAIN& holeOutline = aOutline.Hole( cnt, ii );
             TopoDS_Shape hole;
 
-            if( MakeShape( hole, holeOutline, m_thickness, aOrigin ) )
+            if( MakeShape( hole, holeOutline, m_boardThickness, 0.0, aOrigin ) )
             {
                 m_cutouts.push_back( hole );
             }
@@ -679,6 +727,10 @@ bool STEP_PCB_MODEL::WriteSTEP( const wxString& aFileName )
 
     if( success )
     {
+
+        // Preserve the permissions of the current file
+        KIPLATFORM::IO::DuplicatePermissions( fn.GetFullPath(), tmpfname );
+
         if( !wxRenameFile( tmpfname, fn.GetFullName(), true ) )
         {
             ReportMessage( wxString::Format( wxT( "Cannot rename temporary file '%s' to '%s'.\n" ),
@@ -947,7 +999,7 @@ bool STEP_PCB_MODEL::getModelLocation( bool aBottom, VECTOR2D aPosition, double
     }
     else
     {
-        aOffset.z += m_thickness;
+        aOffset.z += m_boardThickness;
         lRot.SetRotation( gp_Ax1( gp_Pnt( 0.0, 0.0, 0.0 ), gp_Dir( 0.0, 0.0, 1.0 ) ), aRotation );
         lPos.Multiply( lRot );
     }
@@ -1172,4 +1224,4 @@ TDF_Label STEP_PCB_MODEL::transferModel( Handle( TDocStd_Document )& source,
     };
 
     return component;
-}
\ No newline at end of file
+}
