@@ -485,6 +485,7 @@ void FOOTPRINT::GetContextualTextVars( wxArrayString* aVars ) const
     aVars->push_back( wxT( "LAYER" ) );
     aVars->push_back( wxT( "FOOTPRINT_LIBRARY" ) );
     aVars->push_back( wxT( "FOOTPRINT_NAME" ) );
+    aVars->push_back( wxT( "SHORT_NET_NAME(<pad_number>)" ) );
     aVars->push_back( wxT( "NET_NAME(<pad_number>)" ) );
     aVars->push_back( wxT( "NET_CLASS(<pad_number>)" ) );
     aVars->push_back( wxT( "PIN_NAME(<pad_number>)" ) );
@@ -521,9 +522,10 @@ bool FOOTPRINT::ResolveTextVar( wxString* token, int aDepth ) const
         *token = m_fpid.GetLibItemName();
         return true;
     }
-    else if( token->StartsWith( wxT( "NET_NAME(" ) )
-                || token->StartsWith( wxT( "NET_CLASS(" ) )
-                || token->StartsWith( wxT( "PIN_NAME(" ) ) )
+    else if( token->StartsWith( wxT( "SHORT_NET_NAME(" ) )
+                 || token->StartsWith( wxT( "NET_NAME(" ) )
+                 || token->StartsWith( wxT( "NET_CLASS(" ) )
+                 || token->StartsWith( wxT( "PIN_NAME(" ) ) )
     {
         wxString padNumber = token->AfterFirst( '(' );
         padNumber = padNumber.BeforeLast( ')' );
@@ -532,7 +534,9 @@ bool FOOTPRINT::ResolveTextVar( wxString* token, int aDepth ) const
         {
             if( pad->GetNumber() == padNumber )
             {
-                if( token->StartsWith( wxT( "NET_NAME" ) ) )
+                if( token->StartsWith( wxT( "SHORT_NET_NAME" ) ) )
+                    *token = pad->GetShortNetname();
+                else if( token->StartsWith( wxT( "NET_NAME" ) ) )
                     *token = pad->GetNetname();
                 else if( token->StartsWith( wxT( "NET_CLASS" ) ) )
                     *token = pad->GetNetClassName();
@@ -797,7 +801,9 @@ BOX2I FOOTPRINT::GetFpPadsLocalBbox() const
     for( PAD* pad : dummy.Pads() )
         bbox.Merge( pad->GetBoundingBox() );
 
+    // Remove the parent and the group from the dummy footprint before deletion
     dummy.SetParent( nullptr );
+    dummy.SetParentGroup( nullptr );
 
     return bbox;
 }
@@ -941,6 +947,51 @@ const BOX2I FOOTPRINT::GetBoundingBox( bool aIncludeText, bool aIncludeInvisible
 }
 
 
+const BOX2I FOOTPRINT::GetLayerBoundingBox( LSET aLayers ) const
+{
+    std::vector<PCB_TEXT*> texts;
+    const BOARD* board = GetBoard();
+    bool         isFPEdit = board && board->IsFootprintHolder();
+
+    // Start with an uninitialized bounding box
+    BOX2I bbox;
+
+    for( BOARD_ITEM* item : m_drawings )
+    {
+        if( m_privateLayers.test( item->GetLayer() ) && !isFPEdit )
+            continue;
+
+        if( ( aLayers & item->GetLayerSet() ).none() )
+            continue;
+
+        // We want the bitmap bounding box just in the footprint editor
+        // so it will start with the correct initial zoom
+        if( item->Type() == PCB_BITMAP_T && !isFPEdit )
+            continue;
+
+        bbox.Merge( item->GetBoundingBox() );
+    }
+
+    for( PAD* pad : m_pads )
+    {
+        if( ( aLayers & pad->GetLayerSet() ).none() )
+            continue;
+
+        bbox.Merge( pad->GetBoundingBox() );
+    }
+
+    for( ZONE* zone : m_fp_zones )
+    {
+        if( ( aLayers & zone->GetLayerSet() ).none() )
+            continue;
+
+        bbox.Merge( zone->GetBoundingBox() );
+    }
+
+    return bbox;
+}
+
+
 SHAPE_POLY_SET FOOTPRINT::GetBoundingHull() const
 {
     const BOARD* board = GetBoard();
@@ -1097,23 +1148,97 @@ void FOOTPRINT::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_I
 bool FOOTPRINT::IsOnLayer( PCB_LAYER_ID aLayer ) const
 {
     // If we have any pads, fall back on normal checking
-    if( !m_pads.empty() )
-        return m_layer == aLayer;
+    for( PAD* pad : m_pads )
+    {
+        if( pad->IsOnLayer( aLayer ) )
+            return true;
+    }
 
     // No pads?  Check if this entire footprint exists on the given layer
     for( FP_ZONE* zone : m_fp_zones )
     {
-        if( !zone->IsOnLayer( aLayer ) )
-            return false;
+        if( zone->IsOnLayer( aLayer ) )
+            return true;
+    }
+
+    if( m_reference->IsOnLayer( aLayer ) || m_value->IsOnLayer( aLayer ) )
+        return true;
+
+    for( BOARD_ITEM* item : m_drawings )
+    {
+        if( item->IsOnLayer( aLayer ) )
+            return true;
+    }
+
+    return false;
+}
+
+
+bool FOOTPRINT::HitTestOnLayer( const VECTOR2I& aPosition, PCB_LAYER_ID aLayer, int aAccuracy ) const
+{
+    for( PAD* pad : m_pads )
+    {
+        if( pad->IsOnLayer( aLayer ) && pad->HitTest( aPosition, aAccuracy ) )
+            return true;
+    }
+
+    for( ZONE* zone : m_fp_zones )
+    {
+        if( zone->IsOnLayer( aLayer ) && zone->HitTest( aPosition, aAccuracy ) )
+            return true;
     }
 
     for( BOARD_ITEM* item : m_drawings )
     {
-        if( !item->IsOnLayer( aLayer ) )
+        if( item->Type() != PCB_TEXT_T && item->IsOnLayer( aLayer )
+            && item->HitTest( aPosition, aAccuracy ) )
+        {
+            return true;
+        }
+    }
+
+    return false;
+}
+
+
+bool FOOTPRINT::HitTestOnLayer( const BOX2I& aRect, bool aContained, PCB_LAYER_ID aLayer, int aAccuracy ) const
+{
+    std::vector<BOARD_ITEM*> items;
+
+    for( PAD* pad : m_pads )
+    {
+        if( pad->IsOnLayer( aLayer ) )
+            items.push_back( pad );
+    }
+
+    for( FP_ZONE* zone : m_fp_zones )
+    {
+        if( zone->IsOnLayer( aLayer ) )
+            items.push_back( zone );
+    }
+
+    for( BOARD_ITEM* item : m_drawings )
+    {
+        if( item->Type() != PCB_TEXT_T && item->IsOnLayer( aLayer ) )
+            items.push_back( item );
+    }
+
+    // If we require the elements to be contained in the rect and any of them are not,
+    // we can return false;
+    // Conversely, if we just require any of the elements to have a hit, we can return true
+    // when the first one is found.
+    for( BOARD_ITEM* item : items )
+    {
+        if( !aContained && item->HitTest( aRect, aContained, aAccuracy ) )
+            return true;
+        else if( aContained && !item->HitTest( aRect, aContained, aAccuracy ) )
             return false;
     }
 
-    return true;
+    // If we didn't exit in the loop, that means that we did not return false for aContained or
+    // we did not return true for !aContained.  So we can just return the bool with a test of
+    // whether there were any elements or not.
+    return !items.empty() && aContained;
 }
 
 
@@ -1516,7 +1641,7 @@ const BOX2I FOOTPRINT::ViewBBox() const
 
     if( board )
     {
-        int biggest_clearance = board->GetDesignSettings().GetBiggestClearanceValue();
+        int biggest_clearance = board->GetMaxClearanceValue();
         area.Inflate( biggest_clearance );
     }
 
@@ -2547,7 +2672,17 @@ void FOOTPRINT::CheckNetTies( const std::function<void( const BOARD_ITEM* aItem,
     for( BOARD_ITEM* item : m_drawings )
     {
         if( item->IsOnCopperLayer() )
+        {
             copperItems.push_back( item );
+        }
+        else if( PCB_GROUP* group = dynamic_cast<PCB_GROUP*>( item ) )
+        {
+            group->RunOnDescendants( [&]( BOARD_ITEM* descendent )
+                                     {
+                                         if( descendent->IsOnCopperLayer() )
+                                             copperItems.push_back( descendent );
+                                     } );
+        }
     }
 
     for( ZONE* zone : m_fp_zones )
@@ -2859,7 +2994,12 @@ void FOOTPRINT::TransformFPShapesToPolySet( SHAPE_POLY_SET& aBuffer, PCB_LAYER_I
             FP_TEXTBOX* textbox = static_cast<FP_TEXTBOX*>( item );
 
             if( aLayer != UNDEFINED_LAYER && textbox->GetLayer() == aLayer && textbox->IsVisible() )
-                textbox->TransformShapeToPolygon( aBuffer, aLayer, 0, aError, aErrorLoc );
+            {
+                // border
+                textbox->PCB_SHAPE::TransformShapeToPolygon( aBuffer, aLayer, 0, aError, aErrorLoc );
+                // text
+                textbox->TransformTextToPolySet( aBuffer, 0, aError, aErrorLoc );
+            }
         }
 
         if( item->Type() == PCB_FP_SHAPE_T && aIncludeShapes )
