@@ -358,12 +358,12 @@ void PAD::SetChamferRectRatio( double aChamferScale )
 }
 
 
-const std::shared_ptr<SHAPE_POLY_SET>& PAD::GetEffectivePolygon() const
+const std::shared_ptr<SHAPE_POLY_SET>& PAD::GetEffectivePolygon( ERROR_LOC aErrorLoc ) const
 {
-    if( m_polyDirty )
-        BuildEffectivePolygon();
+    if( m_polyDirty[ aErrorLoc ] )
+        BuildEffectivePolygon( aErrorLoc );
 
-    return m_effectivePolygon;
+    return m_effectivePolygon[ aErrorLoc ];
 }
 
 
@@ -416,7 +416,7 @@ std::shared_ptr<SHAPE_SEGMENT> PAD::GetEffectiveHoleShape() const
 int PAD::GetBoundingRadius() const
 {
     if( m_polyDirty )
-        BuildEffectivePolygon();
+        BuildEffectivePolygon( ERROR_OUTSIDE );
 
     return m_effectiveBoundingRadius;
 }
@@ -594,41 +594,46 @@ void PAD::BuildEffectiveShapes( PCB_LAYER_ID aLayer ) const
 }
 
 
-void PAD::BuildEffectivePolygon() const
+void PAD::BuildEffectivePolygon( ERROR_LOC aErrorLoc ) const
 {
     std::lock_guard<std::mutex> RAII_lock( m_polyBuildingLock );
 
     // If we had to wait for the lock then we were probably waiting for someone else to
     // finish rebuilding the shapes.  So check to see if they're clean now.
-    if( !m_polyDirty )
+    if( !m_polyDirty[ aErrorLoc ] )
         return;
 
     const BOARD* board = GetBoard();
     int          maxError = board ? board->GetDesignSettings().m_MaxError : ARC_HIGH_DEF;
 
     // Polygon
-    m_effectivePolygon = std::make_shared<SHAPE_POLY_SET>();
-    TransformShapeToPolygon( *m_effectivePolygon, UNDEFINED_LAYER, 0, maxError, ERROR_INSIDE );
+    std::shared_ptr<SHAPE_POLY_SET>& effectivePolygon = m_effectivePolygon[ aErrorLoc ];
+
+    effectivePolygon = std::make_shared<SHAPE_POLY_SET>();
+    TransformShapeToPolygon( *effectivePolygon, UNDEFINED_LAYER, 0, maxError, aErrorLoc );
 
     // Bounding radius
     //
     // PADSTACKS TODO: these will both need to cycle through all layers to get the largest
     // values....
-    m_effectiveBoundingRadius = 0;
-
-    for( int cnt = 0; cnt < m_effectivePolygon->OutlineCount(); ++cnt )
+    if( aErrorLoc == ERROR_OUTSIDE )
     {
-        const SHAPE_LINE_CHAIN& poly = m_effectivePolygon->COutline( cnt );
+        m_effectiveBoundingRadius = 0;
 
-        for( int ii = 0; ii < poly.PointCount(); ++ii )
+        for( int cnt = 0; cnt < effectivePolygon->OutlineCount(); ++cnt )
         {
-            int dist = KiROUND( ( poly.CPoint( ii ) - m_pos ).EuclideanNorm() );
-            m_effectiveBoundingRadius = std::max( m_effectiveBoundingRadius, dist );
+            const SHAPE_LINE_CHAIN& poly = effectivePolygon->COutline( cnt );
+
+            for( int ii = 0; ii < poly.PointCount(); ++ii )
+            {
+                int dist = KiROUND( ( poly.CPoint( ii ) - m_pos ).EuclideanNorm() );
+                m_effectiveBoundingRadius = std::max( m_effectiveBoundingRadius, dist );
+            }
         }
     }
 
     // All done
-    m_polyDirty = false;
+    m_polyDirty[ aErrorLoc ] = false;
 }
 
 
@@ -942,11 +947,14 @@ VECTOR2I PAD::GetSolderPasteMargin() const
     pad_margin.y = margin + KiROUND( m_size.y * mratio );
 
     // ensure mask have a size always >= 0
-    if( pad_margin.x < -m_size.x / 2 )
-        pad_margin.x = -m_size.x / 2;
+    if( m_padShape != PAD_SHAPE::CUSTOM )
+    {
+        if( pad_margin.x < -m_size.x / 2 )
+            pad_margin.x = -m_size.x / 2;
 
-    if( pad_margin.y < -m_size.y / 2 )
-        pad_margin.y = -m_size.y / 2;
+        if( pad_margin.y < -m_size.y / 2 )
+            pad_margin.y = -m_size.y / 2;
+    }
 
     return pad_margin;
 }
@@ -1101,7 +1109,7 @@ bool PAD::HitTest( const VECTOR2I& aPosition, int aAccuracy ) const
     if( delta.SquaredEuclideanNorm() > SEG::Square( boundingRadius ) )
         return false;
 
-    return GetEffectivePolygon()->Contains( aPosition, -1, aAccuracy );
+    return GetEffectivePolygon( ERROR_INSIDE )->Contains( aPosition, -1, aAccuracy );
 }
 
 
@@ -1124,7 +1132,7 @@ bool PAD::HitTest( const BOX2I& aRect, bool aContained, int aAccuracy ) const
         if( !arect.Intersects( bbox ) )
             return false;
 
-        const std::shared_ptr<SHAPE_POLY_SET>& poly = GetEffectivePolygon();
+        const std::shared_ptr<SHAPE_POLY_SET>& poly = GetEffectivePolygon( ERROR_INSIDE );
 
         int count = poly->TotalVertices();
 
@@ -1482,7 +1490,7 @@ const BOX2I PAD::ViewBBox() const
     if( PCBNEW_SETTINGS* cfg = dynamic_cast<PCBNEW_SETTINGS*>( Kiface().KifaceSettings() ) )
     {
         if( cfg && cfg->m_Display.m_PadClearance && GetBoard() )
-            clearance = GetBoard()->GetDesignSettings().GetBiggestClearanceValue();
+            clearance = GetBoard()->GetMaxClearanceValue();
     }
 
     // Look for the biggest possible bounding box
@@ -1681,9 +1689,9 @@ void PAD::TransformShapeToPolygon( SHAPE_POLY_SET& aBuffer, PCB_LAYER_ID aLayer,
         SHAPE_POLY_SET outline;
         MergePrimitivesAsPolygon( &outline, aErrorLoc );
         outline.Rotate( m_orient );
-        outline.Move( VECTOR2I( m_pos ) );
+        outline.Move( VECTOR2I( padShapePos ) );
 
-        if( aClearance )
+        if( aClearance > 0 )
         {
             int numSegs = std::max( GetArcToSegmentCount( aClearance, aError, FULL_CIRCLE ),
                                                           pad_min_seg_per_circle_count );
@@ -1699,6 +1707,19 @@ void PAD::TransformShapeToPolygon( SHAPE_POLY_SET& aBuffer, PCB_LAYER_ID aLayer,
             outline.Simplify( SHAPE_POLY_SET::PM_FAST );
             outline.Fracture( SHAPE_POLY_SET::PM_FAST );
         }
+        else if( aClearance < 0 )
+        {
+            // Negative clearances are primarily for drawing solder paste layer, so we don't
+            // worry ourselves overly about which side the error is on.
+
+            int numSegs = std::max( GetArcToSegmentCount( aClearance, aError, FULL_CIRCLE ),
+                                                          pad_min_seg_per_circle_count );
+
+            // aClearance is negative so this is actually a deflate
+            outline.Inflate( aClearance, numSegs );
+            outline.Simplify( SHAPE_POLY_SET::PM_FAST );
+            outline.Fracture( SHAPE_POLY_SET::PM_FAST );
+        }
 
         aBuffer.Append( outline );
         break;
@@ -1825,7 +1846,16 @@ static struct PAD_DESC
                     PROPERTY_DISPLAY::PT_SIZE ), groupPad );
         propMgr.AddProperty( new PROPERTY<PAD, int>( _HKI( "Size Y" ),
                     &PAD::SetSizeY, &PAD::GetSizeY,
-                    PROPERTY_DISPLAY::PT_SIZE ), groupPad );
+                    PROPERTY_DISPLAY::PT_SIZE ), groupPad )
+                .SetAvailableFunc(
+                        [=]( INSPECTABLE* aItem ) -> bool
+                        {
+                            // Circle pads have no usable y-size
+                            if( PAD* pad = dynamic_cast<PAD*>( aItem ) )
+                                return pad->GetShape() != PAD_SHAPE::CIRCLE;
+
+                            return true;
+                        } );
 
         auto roundRadiusRatio = new PROPERTY<PAD, double>( _HKI( "Round Radius Ratio" ),
                     &PAD::SetRoundRectRadiusRatio, &PAD::GetRoundRectRadiusRatio );
