@@ -253,9 +253,9 @@ public:
 
     void SetSize( const VECTOR2I& aSize )       { m_size = aSize; SetDirty(); }
     const VECTOR2I& GetSize() const             { return m_size; }
-    void SetSizeX( const int aX )               { m_size.x = aX; SetDirty(); }
+    void SetSizeX( const int aX )               { if( aX > 0 ) { m_size.x = aX; SetDirty(); } }
     int GetSizeX() const                        { return m_size.x; }
-    void SetSizeY( const int aY )               { m_size.y = aY; SetDirty(); }
+    void SetSizeY( const int aY )               { if( aY > 0 ) { m_size.y = aY; SetDirty(); } }
     int GetSizeY() const                        { return m_size.y; }
 
     void SetDelta( const VECTOR2I& aSize )      { m_deltaSize = aSize; SetDirty(); }
@@ -382,13 +382,14 @@ public:
 
     bool IsDirty() const
     {
-        return m_shapesDirty || m_polyDirty;
+        return m_shapesDirty || m_polyDirty[ERROR_INSIDE] || m_polyDirty[ERROR_OUTSIDE];
     }
 
     void SetDirty()
     {
         m_shapesDirty = true;
-        m_polyDirty = true;
+        m_polyDirty[ERROR_INSIDE] = true;
+        m_polyDirty[ERROR_OUTSIDE] = true;
     }
 
     void SetLayerSet( LSET aLayers ) override   { m_layerMask = aLayers; }
@@ -469,7 +470,7 @@ public:
     GetEffectiveShape( PCB_LAYER_ID aLayer = UNDEFINED_LAYER,
                        FLASHING flashPTHPads = FLASHING::DEFAULT ) const override;
 
-    const std::shared_ptr<SHAPE_POLY_SET>& GetEffectivePolygon() const;
+    const std::shared_ptr<SHAPE_POLY_SET>& GetEffectivePolygon( ERROR_LOC aErrorLoc ) const;
 
     /**
      * Return a SHAPE_SEGMENT object representing the pad's hole.
@@ -729,7 +730,7 @@ public:
      * the dirty bit.
      */
     void BuildEffectiveShapes( PCB_LAYER_ID aLayer ) const;
-    void BuildEffectivePolygon() const;
+    void BuildEffectivePolygon( ERROR_LOC aErrorLoc ) const;
 
     virtual void ViewGetLayers( int aLayers[], int& aCount ) const override;
 
@@ -744,13 +745,13 @@ public:
 
     const ZONE_LAYER_OVERRIDE& GetZoneLayerOverride( PCB_LAYER_ID aLayer ) const
     {
-        return m_zoneLayerOverrides[ aLayer ];
+        return m_zoneLayerOverrides.at( aLayer );
     }
 
     void SetZoneLayerOverride( PCB_LAYER_ID aLayer, ZONE_LAYER_OVERRIDE aOverride )
     {
         std::unique_lock<std::mutex> cacheLock( m_zoneLayerOverridesMutex );
-        m_zoneLayerOverrides[ aLayer ] = aOverride;
+        m_zoneLayerOverrides.at( aLayer ) = aOverride;
     }
 
 #if defined(DEBUG)
@@ -787,9 +788,9 @@ private:
     mutable std::shared_ptr<SHAPE_COMPOUND>   m_effectiveShape;
     mutable std::shared_ptr<SHAPE_SEGMENT>    m_effectiveHoleShape;
 
-    mutable bool                              m_polyDirty;
+    mutable bool                              m_polyDirty[2];
     mutable std::mutex                        m_polyBuildingLock;
-    mutable std::shared_ptr<SHAPE_POLY_SET>   m_effectivePolygon;
+    mutable std::shared_ptr<SHAPE_POLY_SET>   m_effectivePolygon[2];
     mutable int                               m_effectiveBoundingRadius;
 
     /*
