@@ -1111,7 +1111,7 @@ void PCB_PAINTER::draw( const PAD* aPad, int aLayer )
         VECTOR2D position = padBBox.Centre();
         VECTOR2D padsize = VECTOR2D( padBBox.GetSize() );
 
-        if( aPad->GetFlags() & ENTERED )
+        if( aPad->IsEntered() )
         {
             FOOTPRINT* fp = static_cast<FOOTPRINT*>( aPad->GetParentFootprint() );
 
@@ -1623,8 +1623,6 @@ void PCB_PAINTER::draw( const PAD* aPad, int aLayer )
             }
             else if( aPad->GetEffectiveHoleShape() && clearance > 0 )
             {
-                clearance += m_holePlatingThickness;
-
                 std::shared_ptr<SHAPE_SEGMENT> slot = aPad->GetEffectiveHoleShape();
                 m_gal->DrawSegment( slot->GetSeg().A, slot->GetSeg().B,
                                     slot->GetWidth() + 2 * clearance );
@@ -1884,7 +1882,23 @@ void PCB_PAINTER::strokeText( const wxString& aText, const VECTOR2I& aPosition,
     m_gal->SetIsFill( font->IsOutline() );
     m_gal->SetIsStroke( font->IsStroke() );
 
-    font->Draw( m_gal, aText, aPosition, aAttrs );
+    VECTOR2I pos( aPosition );
+    VECTOR2I fudge
+    { KiROUND( 0.16 * aAttrs.m_StrokeWidth ), 0 };
+    RotatePoint( fudge, aAttrs.m_Angle );
+
+    if( ( aAttrs.m_Halign == GR_TEXT_H_ALIGN_LEFT && !aAttrs.m_Mirrored )
+            || ( aAttrs.m_Halign == GR_TEXT_H_ALIGN_RIGHT && aAttrs.m_Mirrored ) )
+    {
+        pos -= fudge;
+    }
+    else if( ( aAttrs.m_Halign == GR_TEXT_H_ALIGN_RIGHT && !aAttrs.m_Mirrored )
+            || ( aAttrs.m_Halign == GR_TEXT_H_ALIGN_LEFT && aAttrs.m_Mirrored ) )
+    {
+        pos += fudge;
+    }
+
+    font->Draw( m_gal, aText, pos, aAttrs );
 }
 
 
@@ -1937,7 +1951,7 @@ void PCB_PAINTER::draw( const PCB_BITMAP* aBitmap, int aLayer )
 
 void PCB_PAINTER::draw( const PCB_TEXT* aText, int aLayer )
 {
-    wxString resolvedText( aText->GetShownText() );
+    wxString resolvedText( aText->GetShownText( true ) );
 
     if( resolvedText.Length() == 0 )
         return;
@@ -1976,29 +1990,8 @@ void PCB_PAINTER::draw( const PCB_TEXT* aText, int aLayer )
 
     if( aText->IsKnockout() )
     {
-        KIGFX::GAL_DISPLAY_OPTIONS empty_opts;
-        SHAPE_POLY_SET             knockouts;
-
-        CALLBACK_GAL callback_gal( empty_opts,
-                // Polygon callback
-                [&]( const SHAPE_LINE_CHAIN& aPoly )
-                {
-                    knockouts.AddOutline( aPoly );
-                } );
-
-        attrs.m_StrokeWidth = getLineThickness( aText->GetEffectiveTextPenWidth() );
-
-        callback_gal.SetIsFill( font->IsOutline() );
-        callback_gal.SetIsStroke( font->IsStroke() );
-        callback_gal.SetLineWidth( attrs.m_StrokeWidth );
-        font->Draw( &callback_gal, resolvedText, aText->GetDrawPos(), attrs );
-
         SHAPE_POLY_SET finalPoly;
-        int            margin = attrs.m_StrokeWidth * 1.5
-                                    + GetKnockoutTextMargin( attrs.m_Size, attrs.m_StrokeWidth );
-
-        aText->TransformBoundingBoxToPolygon( &finalPoly, margin );
-        finalPoly.BooleanSubtract( knockouts, SHAPE_POLY_SET::PM_FAST );
+        aText->TransformTextToPolySet( finalPoly, 0, m_maxError, ERROR_INSIDE );
         finalPoly.Fracture( SHAPE_POLY_SET::PM_FAST );
 
         m_gal->SetIsStroke( false );
@@ -2024,9 +2017,14 @@ void PCB_PAINTER::draw( const PCB_TEXT* aText, int aLayer )
             cache = aText->GetRenderCache( font, resolvedText );
 
         if( cache )
+        {
+            m_gal->SetLineWidth( attrs.m_StrokeWidth );
             m_gal->DrawGlyphs( *cache );
+        }
         else
+        {
             strokeText( resolvedText, aText->GetTextPos(), attrs );
+        }
     }
 }
 
@@ -2036,7 +2034,7 @@ void PCB_PAINTER::draw( const PCB_TEXTBOX* aTextBox, int aLayer )
     const COLOR4D& color = m_pcbSettings.GetColor( aTextBox, aLayer );
     int            thickness = getLineThickness( aTextBox->GetWidth() );
     PLOT_DASH_TYPE lineStyle = aTextBox->GetStroke().GetPlotStyle();
-    wxString       resolvedText( aTextBox->GetShownText() );
+    wxString       resolvedText( aTextBox->GetShownText( true ) );
 
     KIFONT::FONT* font = aTextBox->GetFont();
 
@@ -2122,15 +2120,20 @@ void PCB_PAINTER::draw( const PCB_TEXTBOX* aTextBox, int aLayer )
         cache = aTextBox->GetRenderCache( font, resolvedText );
 
     if( cache )
+    {
+        m_gal->SetLineWidth( attrs.m_StrokeWidth );
         m_gal->DrawGlyphs( *cache );
+    }
     else
+    {
         strokeText( resolvedText, aTextBox->GetDrawPos(), attrs );
+    }
 }
 
 
 void PCB_PAINTER::draw( const FP_TEXT* aText, int aLayer )
 {
-    wxString resolvedText( aText->GetShownText() );
+    wxString resolvedText( aText->GetShownText( true ) );
 
     if( resolvedText.Length() == 0 )
         return;
@@ -2153,29 +2156,8 @@ void PCB_PAINTER::draw( const FP_TEXT* aText, int aLayer )
 
     if( aText->IsKnockout() )
     {
-        KIGFX::GAL_DISPLAY_OPTIONS empty_opts;
-        SHAPE_POLY_SET             knockouts;
-
-        CALLBACK_GAL callback_gal( empty_opts,
-                // Polygon callback
-                [&]( const SHAPE_LINE_CHAIN& aPoly )
-                {
-                    knockouts.AddOutline( aPoly );
-                } );
-
-        attrs.m_StrokeWidth = getLineThickness( aText->GetEffectiveTextPenWidth() );
-
-        callback_gal.SetIsFill( font->IsOutline() );
-        callback_gal.SetIsStroke( font->IsStroke() );
-        callback_gal.SetLineWidth( attrs.m_StrokeWidth );
-        font->Draw( &callback_gal, resolvedText, aText->GetDrawPos(), attrs );
-
         SHAPE_POLY_SET finalPoly;
-        int            margin = attrs.m_StrokeWidth * 1.5
-                                    + GetKnockoutTextMargin( attrs.m_Size, attrs.m_StrokeWidth );
-
-        aText->TransformBoundingBoxToPolygon( &finalPoly, margin );
-        finalPoly.BooleanSubtract( knockouts, SHAPE_POLY_SET::PM_FAST );
+        aText->TransformTextToPolySet( finalPoly, 0, m_maxError, ERROR_INSIDE );
         finalPoly.Fracture( SHAPE_POLY_SET::PM_FAST );
 
         m_gal->SetIsStroke( false );
@@ -2201,9 +2183,14 @@ void PCB_PAINTER::draw( const FP_TEXT* aText, int aLayer )
             cache = aText->GetRenderCache( font, resolvedText );
 
         if( cache )
+        {
+            m_gal->SetLineWidth( attrs.m_StrokeWidth );
             m_gal->DrawGlyphs( *cache );
+        }
         else
+        {
             strokeText( resolvedText, aText->GetTextPos(), attrs );
+        }
     }
 
     // Draw the umbilical line
@@ -2254,7 +2241,7 @@ void PCB_PAINTER::draw( const FP_TEXTBOX* aTextBox, int aLayer )
         }
     }
 
-    wxString resolvedText( aTextBox->GetShownText() );
+    wxString resolvedText( aTextBox->GetShownText( true ) );
 
     if( resolvedText.Length() == 0 )
         return;
@@ -2275,9 +2262,14 @@ void PCB_PAINTER::draw( const FP_TEXTBOX* aTextBox, int aLayer )
         cache = aTextBox->GetRenderCache( aTextBox->GetFont(), resolvedText );
 
     if( cache )
+    {
+        m_gal->SetLineWidth( attrs.m_StrokeWidth );
         m_gal->DrawGlyphs( *cache );
+    }
     else
+    {
         strokeText( resolvedText, aTextBox->GetDrawPos(), attrs );
+    }
 }
 
 
@@ -2454,8 +2446,9 @@ void PCB_PAINTER::draw( const ZONE* aZone, int aLayer )
     if( !IsZoneFillLayer( aLayer ) )
     {
         const SHAPE_POLY_SET* outline = aZone->Outline();
+        bool allowDrawOutline = aZone->GetHatchStyle() != ZONE_BORDER_DISPLAY_STYLE::INVISIBLE_BORDER;
 
-        if( !m_pcbSettings.m_isPrinting && outline && outline->OutlineCount() > 0 )
+        if( allowDrawOutline && !m_pcbSettings.m_isPrinting && outline && outline->OutlineCount() > 0 )
         {
             m_gal->SetStrokeColor( color.a > 0.0 ? color.WithAlpha( 1.0 ) : color );
             m_gal->SetIsFill( false );
@@ -2561,7 +2554,7 @@ void PCB_PAINTER::draw( const PCB_DIMENSION_BASE* aDimension, int aLayer )
     }
 
     // Draw text
-    wxString        resolvedText = aDimension->GetShownText();
+    wxString        resolvedText = aDimension->GetShownText( true );
     TEXT_ATTRIBUTES attrs = aDimension->GetAttributes();
 
     if( m_gal->IsFlippedX() && !( aDimension->GetLayerSet() & LSET::SideSpecificMask() ).any() )
