@@ -97,7 +97,7 @@ int PCBNEW_JOBS_HANDLER::JobExportStep( JOB* aJob )
 
     EXPORTER_STEP_PARAMS params;
     params.m_includeExcludedBom = aStepJob->m_includeExcludedBom;
-    params.m_minDistance = aStepJob->m_minDistance;
+    params.m_boardOutlinesChainingEpsilon = aStepJob->m_boardOutlinesChainingEpsilon;
     params.m_overwrite = aStepJob->m_overwrite;
     params.m_substModels = aStepJob->m_substModels;
     params.m_origin = VECTOR2D( aStepJob->m_xOrigin, aStepJob->m_yOrigin );
@@ -129,9 +129,11 @@ int PCBNEW_JOBS_HANDLER::JobExportSvg( JOB* aJob )
     svgPlotOptions.m_colorTheme = aSvgJob->m_colorTheme;
     svgPlotOptions.m_outputFile = aSvgJob->m_outputFile;
     svgPlotOptions.m_mirror = aSvgJob->m_mirror;
+    svgPlotOptions.m_negative = aSvgJob->m_negative;
     svgPlotOptions.m_pageSizeMode = aSvgJob->m_pageSizeMode;
     svgPlotOptions.m_printMaskLayer = aSvgJob->m_printMaskLayer;
     svgPlotOptions.m_plotFrame = aSvgJob->m_plotDrawingSheet;
+    svgPlotOptions.m_drillShapeOption = aSvgJob->m_drillShapeOption;
 
     if( aJob->IsCli() )
         wxPrintf( _( "Loading board\n" ) );
@@ -237,7 +239,25 @@ int PCBNEW_JOBS_HANDLER::JobExportPdf( JOB* aJob )
 
     SETTINGS_MANAGER& mgr = Pgm().GetSettingsManager();
     plotOpts.SetColorSettings( mgr.GetColorSettings( aPdfJob->m_colorTheme ) );
+    plotOpts.SetMirror( aPdfJob->m_mirror );
     plotOpts.SetBlackAndWhite( aPdfJob->m_blackAndWhite );
+    plotOpts.SetNegative( aPdfJob->m_negative );
+
+    switch( aPdfJob->m_drillShapeOption )
+    {
+        default:
+        case 0:
+            plotOpts.SetDrillMarksType( DRILL_MARKS::NO_DRILL_SHAPE );
+            break;
+
+        case 1:
+            plotOpts.SetDrillMarksType( DRILL_MARKS::SMALL_DRILL_SHAPE );
+            break;
+
+        case 2:
+            plotOpts.SetDrillMarksType( DRILL_MARKS::FULL_DRILL_SHAPE );
+            break;
+    }
 
     PDF_PLOTTER* plotter = (PDF_PLOTTER*) StartPlotBoard(
             brd, &plotOpts, UNDEFINED_LAYER, aPdfJob->m_outputFile, wxEmptyString, wxEmptyString );
@@ -257,6 +277,7 @@ int PCBNEW_JOBS_HANDLER::JobExportPdf( JOB* aJob )
 
 int PCBNEW_JOBS_HANDLER::JobExportGerbers( JOB* aJob )
 {
+    int                     exitCode = CLI::EXIT_CODES::OK;
     JOB_EXPORT_PCB_GERBERS* aGerberJob = dynamic_cast<JOB_EXPORT_PCB_GERBERS*>( aJob );
 
     if( aGerberJob == nullptr )
@@ -307,8 +328,6 @@ int PCBNEW_JOBS_HANDLER::JobExportGerbers( JOB* aJob )
         // Pick the basename from the board file
         wxFileName fn( brd->GetFileName() );
         PCB_LAYER_ID layer = *seq;
-        fileExt = GetGerberProtelExtension( layer );
-
         PCB_PLOT_PARAMS plotOpts;
 
         if( aGerberJob->m_useBoardPlotParams )
@@ -316,6 +335,11 @@ int PCBNEW_JOBS_HANDLER::JobExportGerbers( JOB* aJob )
         else
             populateGerberPlotOptionsFromJob( plotOpts, aGerberJob );
 
+        if( plotOpts.GetUseGerberProtelExtensions() )
+            fileExt = GetGerberProtelExtension( layer );
+        else
+            fileExt = GerberFileExtension;
+
         BuildPlotFileName( &fn, aGerberJob->m_outputFile, brd->GetLayerName( layer ), fileExt );
         wxString fullname = fn.GetFullName();
 
@@ -331,16 +355,22 @@ int PCBNEW_JOBS_HANDLER::JobExportGerbers( JOB* aJob )
             PlotBoardLayers( brd, plotter, plotSequence, plotOpts );
             plotter->EndPlot();
         }
+        else
+        {
+            wxFprintf( stderr, _( "Failed to plot to '%s'.\n" ), fn.GetFullPath() );
+            exitCode = CLI::EXIT_CODES::ERR_INVALID_OUTPUT_CONFLICT;
+        }
 
         delete plotter;
     }
 
     wxFileName fn( aGerberJob->m_filename );
+
     // Build gerber job file from basename
     BuildPlotFileName( &fn, aGerberJob->m_outputFile, wxT( "job" ), GerberJobFileExtension );
     jobfile_writer.CreateJobFile( fn.GetFullPath() );
 
-    return CLI::EXIT_CODES::OK;
+    return exitCode;
 }
 
 
@@ -361,13 +391,15 @@ void PCBNEW_JOBS_HANDLER::populateGerberPlotOptionsFromJob( PCB_PLOT_PARAMS&
     aPlotOpts.SetDisableGerberMacros( aJob->m_disableApertureMacros );
     aPlotOpts.SetUseGerberX2format( aJob->m_useX2Format );
     aPlotOpts.SetIncludeGerberNetlistInfo( aJob->m_includeNetlistAttributes );
-
+    aPlotOpts.SetUseAuxOrigin( aJob->m_useAuxOrigin );
+    aPlotOpts.SetUseGerberProtelExtensions( aJob->m_useProtelFileExtension );
     aPlotOpts.SetGerberPrecision( aJob->m_precision );
 }
 
 
 int PCBNEW_JOBS_HANDLER::JobExportGerber( JOB* aJob )
 {
+    int                    exitCode = CLI::EXIT_CODES::OK;
     JOB_EXPORT_PCB_GERBER* aGerberJob = dynamic_cast<JOB_EXPORT_PCB_GERBER*>( aJob );
 
     if( aGerberJob == nullptr )
@@ -402,10 +434,15 @@ int PCBNEW_JOBS_HANDLER::JobExportGerber( JOB* aJob )
                          plotOpts );
         plotter->EndPlot();
     }
+    else
+    {
+        wxFprintf( stderr, _( "Failed to plot to '%s'.\n" ), aGerberJob->m_outputFile );
+        exitCode = CLI::EXIT_CODES::ERR_INVALID_OUTPUT_CONFLICT;
+    }
 
     delete plotter;
 
-    return CLI::EXIT_CODES::OK;
+    return exitCode;
 }
 
 static DRILL_PRECISION precisionListForInches( 2, 4 );
@@ -495,8 +532,11 @@ int PCBNEW_JOBS_HANDLER::JobExportDrill( JOB* aJob )
         excellonWriter->SetRouteModeForOvalHoles( aDrillJob->m_excellonOvalDrillRoute );
         excellonWriter->SetMapFileFormat( mapFormat );
 
-        excellonWriter->CreateDrillandMapFilesSet( aDrillJob->m_outputDir, true,
-                                                   aDrillJob->m_generateMap, nullptr );
+        if( !excellonWriter->CreateDrillandMapFilesSet( aDrillJob->m_outputDir, true,
+                                                        aDrillJob->m_generateMap, this ) )
+        {
+            return CLI::EXIT_CODES::ERR_INVALID_OUTPUT_CONFLICT;
+        }
     }
     else if( aDrillJob->m_format == JOB_EXPORT_PCB_DRILL::DRILL_FORMAT::GERBER )
     {
@@ -512,8 +552,11 @@ int PCBNEW_JOBS_HANDLER::JobExportDrill( JOB* aJob )
         gerberWriter->SetOptions( offset );
         gerberWriter->SetMapFileFormat( mapFormat );
 
-        gerberWriter->CreateDrillandMapFilesSet( aDrillJob->m_outputDir, true,
-                                                 aDrillJob->m_generateMap, nullptr );
+        if( !gerberWriter->CreateDrillandMapFilesSet( aDrillJob->m_outputDir, true,
+                                                      aDrillJob->m_generateMap, this ) )
+        {
+            return CLI::EXIT_CODES::ERR_INVALID_OUTPUT_CONFLICT;
+        }
     }
 
     return CLI::EXIT_CODES::OK;
@@ -780,3 +823,14 @@ int PCBNEW_JOBS_HANDLER::doFpExportSvg( JOB_FP_EXPORT_SVG* aSvgJob, const FOOTPR
 
     return CLI::EXIT_CODES::OK;
 }
+
+
+REPORTER& PCBNEW_JOBS_HANDLER::Report( const wxString& aText, SEVERITY aSeverity )
+{
+    if( aSeverity == RPT_SEVERITY_ERROR )
+        wxFprintf( stderr, wxS( "%s\n" ), aText );
+    else
+        wxPrintf( wxS( "%s\n" ), aText );
+
+    return *this;
+}
