@@ -83,11 +83,11 @@ void PlotInteractiveLayer( BOARD* aBoard, PLOTTER* aPlotter )
 
         properties.emplace_back( wxString::Format( wxT( "!%s = %s" ),
                                                    _( "Reference designator" ),
-                                                   fp->Reference().GetShownText() ) );
+                                                   fp->Reference().GetShownText( false ) ) );
 
         properties.emplace_back( wxString::Format( wxT( "!%s = %s" ),
                                                    _( "Value" ),
-                                                   fp->Value().GetShownText() ) );
+                                                   fp->Value().GetShownText( false ) ) );
 
         for( const auto& [ name, value ] : fp->GetProperties() )
             properties.emplace_back( wxString::Format( wxT( "!%s = %s" ), name, value ) );
@@ -638,17 +638,28 @@ void PlotStandardLayer( BOARD* aBoard, PLOTTER* aPlotter, LSET aLayerMask,
         {
             const     PCB_ARC* arc = static_cast<const PCB_ARC*>( track );
 
-            // ThickArc expects only positive angle arcs, so flip start/end if
-            // we are negative
-            if( arc->GetAngle() < ANGLE_0 )
+            // Too small arcs cannot be really handled: arc center (and arc radius)
+            // cannot be safely computed
+            if( arc->IsDegenerated( 10 /* in IU */ ) )
             {
-                aPlotter->ThickArc( arc->GetCenter(), arc->GetEnd(), arc->GetStart(),
-                                    width, plotMode, &gbr_metadata );
+                // Approximate this very small arc by a segment.
+                aPlotter->ThickSegment( track->GetStart(), track->GetEnd(), width, plotMode,
+                                        &gbr_metadata );
             }
             else
             {
-                aPlotter->ThickArc( arc->GetCenter(), arc->GetStart(), arc->GetEnd(),
-                                    width, plotMode, &gbr_metadata );
+                // ThickArc expects only positive angle arcs, so flip start/end if
+                // we are negative
+                if( arc->GetAngle() < ANGLE_0 )
+                {
+                    aPlotter->ThickArc( arc->GetCenter(), arc->GetEnd(), arc->GetStart(),
+                                        width, plotMode, &gbr_metadata );
+                }
+                else
+                {
+                    aPlotter->ThickArc( arc->GetCenter(), arc->GetStart(), arc->GetEnd(),
+                                        width, plotMode, &gbr_metadata );
+                }
             }
         }
         else
@@ -854,10 +865,10 @@ void PlotSolderMaskLayer( BOARD *aBoard, PLOTTER* aPlotter, LSET aLayerMask,
                     return;
 
                 // add shapes with their exact mask layer size in initialPolys
-                aText.TransformTextToPolySet( initialPolys, layer, 0, maxError, ERROR_OUTSIDE );
+                aText.TransformTextToPolySet( initialPolys, 0, maxError, ERROR_OUTSIDE );
 
                 // add shapes inflated by aMinThickness/2 in areas
-                aText.TransformTextToPolySet( areas, layer, inflate, maxError, ERROR_OUTSIDE );
+                aText.TransformTextToPolySet( areas, inflate, maxError, ERROR_OUTSIDE );
             };
 
     // Generate polygons with arcs inside the shape or exact shape to minimize shape changes
@@ -897,10 +908,6 @@ void PlotSolderMaskLayer( BOARD *aBoard, PLOTTER* aPlotter, LSET aLayerMask,
                                                        ERROR_OUTSIDE );
                     }
                 }
-                else if( item->IsOnLayer( Edge_Cuts ) )
-                {
-                   itemplotter.PlotFootprintShape( static_cast<const FP_SHAPE*>( item ) );
-                }
             }
         }
 
@@ -939,10 +946,10 @@ void PlotSolderMaskLayer( BOARD *aBoard, PLOTTER* aPlotter, LSET aLayerMask,
                     const PCB_TEXT* text = static_cast<const PCB_TEXT*>( item );
 
                     // add shapes with their exact mask layer size in initialPolys
-                    text->TransformTextToPolySet( initialPolys, layer, 0, maxError, ERROR_OUTSIDE );
+                    text->TransformTextToPolySet( initialPolys, 0, maxError, ERROR_OUTSIDE );
 
                     // add shapes inflated by aMinThickness/2 in areas
-                    text->TransformTextToPolySet( areas, layer, inflate, maxError, ERROR_OUTSIDE );
+                    text->TransformTextToPolySet( areas, inflate, maxError, ERROR_OUTSIDE );
                 }
                 else
                 {
@@ -955,10 +962,6 @@ void PlotSolderMaskLayer( BOARD *aBoard, PLOTTER* aPlotter, LSET aLayerMask,
                                                    ERROR_OUTSIDE );
                 }
             }
-            else if( item->IsOnLayer( Edge_Cuts ) )
-            {
-                itemplotter.PlotPcbGraphicItem( item );
-            }
         }
 
         for( ZONE* zone : aBoard->Zones() )
@@ -1226,29 +1229,30 @@ PLOTTER* StartPlotBoard( BOARD *aBoard, const PCB_PLOT_PARAMS *aPlotOpts, int aL
             AddGerberX2Attribute( plotter, aBoard, aLayer, not useX2mode );
         }
 
-        plotter->StartPlot( wxT( "1" ) );
-
-        // Plot the frame reference if requested
-        if( aPlotOpts->GetPlotFrameRef() )
+        if( plotter->StartPlot( wxT( "1" ) ) )
         {
-            PlotDrawingSheet( plotter, aBoard->GetProject(), aBoard->GetTitleBlock(),
-                              aBoard->GetPageSettings(), &aBoard->GetProperties(), wxT( "1" ), 1,
-                              aSheetName, aSheetPath, aBoard->GetFileName() );
+            // Plot the frame reference if requested
+            if( aPlotOpts->GetPlotFrameRef() )
+            {
+                PlotDrawingSheet( plotter, aBoard->GetProject(), aBoard->GetTitleBlock(),
+                                aBoard->GetPageSettings(), &aBoard->GetProperties(), wxT( "1" ), 1,
+                                aSheetName, aSheetPath, aBoard->GetFileName() );
 
-            if( aPlotOpts->GetMirror() )
-                initializePlotter( plotter, aBoard, aPlotOpts );
+                if( aPlotOpts->GetMirror() )
+                    initializePlotter( plotter, aBoard, aPlotOpts );
+            }
+
+            // When plotting a negative board: draw a black rectangle (background for plot board
+            // in white) and switch the current color to WHITE; note the color inversion is actually
+            // done in the driver (if supported)
+            if( aPlotOpts->GetNegative() )
+            {
+                BOX2I bbox = aBoard->ComputeBoundingBox();
+                FillNegativeKnockout( plotter, bbox );
+            }
+
+            return plotter;
         }
-
-        // When plotting a negative board: draw a black rectangle (background for plot board
-        // in white) and switch the current color to WHITE; note the color inversion is actually
-        // done in the driver (if supported)
-        if( aPlotOpts->GetNegative() )
-        {
-            BOX2I bbox = aBoard->ComputeBoundingBox();
-            FillNegativeKnockout( plotter, bbox );
-        }
-
-        return plotter;
     }
 
     delete plotter->RenderSettings();
