@@ -2,6 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2020 Thomas Pointhuber <thomas.pointhuber@gmx.at>
+ * Copyright (C) 2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -272,7 +273,14 @@ ABOARD6::ABOARD6( ALTIUM_PARSER& aReader )
 
         ABOARD6_LAYER_STACKUP l;
 
-        l.name   = ALTIUM_PARSER::ReadString( props, layername, wxT( "" ) );
+        l.name = ALTIUM_PARSER::ReadString( props, layername, wxT( "" ) );
+        wxString originalName = l.name;
+        int ii = 2;
+
+        // Ensure that layer names are unique in KiCad
+        while( !layerNames.insert( l.name ).second )
+            l.name = wxString::Format( wxT( "%s %d" ), originalName, ii++ );
+
         l.nextId = ALTIUM_PARSER::ReadInt( props, layeri + wxT( "NEXT" ), 0 );
         l.prevId = ALTIUM_PARSER::ReadInt( props, layeri + wxT( "PREV" ), 0 );
         l.copperthick = ALTIUM_PARSER::ReadKicadUnit( props, layeri + wxT( "COPTHICK" ), wxT( "1.4mil" ) );
@@ -330,8 +338,10 @@ ACOMPONENT6::ACOMPONENT6( ALTIUM_PARSER& aReader )
     nameon           = ALTIUM_PARSER::ReadBool( props, wxT( "NAMEON" ), true );
     commenton        = ALTIUM_PARSER::ReadBool( props, wxT( "COMMENTON" ), false );
     sourcedesignator = ALTIUM_PARSER::ReadString( props, wxT( "SOURCEDESIGNATOR" ), wxT( "" ) );
-    sourcefootprintlibrary = ALTIUM_PARSER::ReadString( props, wxT( "SOURCEFOOTPRINTLIBRARY" ), wxT( "" ) );
-    pattern = ALTIUM_PARSER::ReadString( props, wxT( "PATTERN" ), wxT( "" ) );
+
+    sourcefootprintlibrary =
+            ALTIUM_PARSER::ReadUnicodeString( props, wxT( "SOURCEFOOTPRINTLIBRARY" ), wxT( "" ) );
+    pattern = ALTIUM_PARSER::ReadUnicodeString( props, wxT( "PATTERN" ), wxT( "" ) );
 
     sourcecomponentlibrary = ALTIUM_PARSER::ReadString( props, wxT( "SOURCECOMPONENTLIBRARY" ), wxT( "" ) );
     sourcelibreference = ALTIUM_PARSER::ReadString( props, wxT( "SOURCELIBREFERENCE" ), wxT( "" ) );
@@ -533,6 +543,13 @@ ARULE6::ARULE6( ALTIUM_PARSER& aReader )
     else if( rulekind == wxT( "PasteMaskExpansion" ) )
     {
         kind = ALTIUM_RULE_KIND::PASTE_MASK_EXPANSION;
+        pastemaskExpansion = ALTIUM_PARSER::ReadKicadUnit( props, wxT( "EXPANSION" ), wxT( "0" ) );
+    }
+    else if( rulekind == wxT( "SolderMaskExpansion" ) )
+    {
+        kind = ALTIUM_RULE_KIND::SOLDER_MASK_EXPANSION;
+        soldermaskExpansion =
+                ALTIUM_PARSER::ReadKicadUnit( props, wxT( "EXPANSION" ), wxT( "4mil" ) );
     }
     else if( rulekind == wxT( "PlaneClearance" ) )
     {
@@ -1040,7 +1057,7 @@ AREGION6::AREGION6( ALTIUM_PARSER& aReader, bool aExtendedVertices )
         kind = ALTIUM_REGION_KIND::POLYGON_CUTOUT;
         break;
     case 2:
-        kind = ALTIUM_REGION_KIND::UNKNOWN_2; // TODO: what kind is this?
+        kind = ALTIUM_REGION_KIND::DASHED_OUTLINE;
         break;
     case 3:
         kind = ALTIUM_REGION_KIND::UNKNOWN_3; // TODO: what kind is this?
@@ -1055,6 +1072,9 @@ AREGION6::AREGION6( ALTIUM_PARSER& aReader, bool aExtendedVertices )
 
     uint32_t num_outline_vertices = aReader.Read<uint32_t>();
 
+    if( aExtendedVertices )
+        num_outline_vertices++; // Has a closing vertex
+
     for( uint32_t i = 0; i < num_outline_vertices; i++ )
     {
         if( aExtendedVertices )
@@ -1076,21 +1096,17 @@ AREGION6::AREGION6( ALTIUM_PARSER& aReader, bool aExtendedVertices )
         }
     }
 
-    // TODO: for now we only support holes in regions where there are stored as double
-    if( !aExtendedVertices )
+    holes.resize( holecount );
+    for( uint16_t k = 0; k < holecount; k++ )
     {
-        holes.resize( holecount );
-        for( uint16_t k = 0; k < holecount; k++ )
-        {
-            uint32_t num_hole_vertices = aReader.Read<uint32_t>();
-            holes.at( k ).reserve( num_hole_vertices );
+        uint32_t num_hole_vertices = aReader.Read<uint32_t>();
+        holes.at( k ).reserve( num_hole_vertices );
 
-            for( uint32_t i = 0; i < num_hole_vertices; i++ )
-            {
-                int32_t x = ALTIUM_PARSER::ConvertToKicadUnit( aReader.Read<double>() );
-                int32_t y = ALTIUM_PARSER::ConvertToKicadUnit( -aReader.Read<double>() );
-                holes.at( k ).emplace_back( VECTOR2I( x, y ) );
-            }
+        for( uint32_t i = 0; i < num_hole_vertices; i++ )
+        {
+            int32_t x = ALTIUM_PARSER::ConvertToKicadUnit( aReader.Read<double>() );
+            int32_t y = ALTIUM_PARSER::ConvertToKicadUnit( -aReader.Read<double>() );
+            holes.at( k ).emplace_back( VECTOR2I( x, y ) );
         }
     }
 
