@@ -1157,8 +1157,8 @@ std::unique_ptr<PNS::SOLID> PNS_KICAD_IFACE_BASE::syncPad( PAD* aPad )
 
     // We generate a single SOLID for a pad, so we have to treat it as ALWAYS_FLASHED and then
     // perform layer-specific flashing tests internally.
-    std::shared_ptr<SHAPE> shape = aPad->GetEffectiveShape( UNDEFINED_LAYER,
-                                                            FLASHING::ALWAYS_FLASHED );
+    const std::shared_ptr<SHAPE>& shape = aPad->GetEffectiveShape( UNDEFINED_LAYER,
+                                                                   FLASHING::ALWAYS_FLASHED );
 
     if( shape->HasIndexableSubshapes() && shape->GetIndexableSubshapeCount() == 1 )
     {
@@ -1167,9 +1167,14 @@ std::unique_ptr<PNS::SOLID> PNS_KICAD_IFACE_BASE::syncPad( PAD* aPad )
 
         solid->SetShape( subshapes[0]->Clone() );
     }
+    // For anything that's not a single shape we use a polygon.  Multiple shapes have a tendency
+    // to confuse the hull generator. https://gitlab.com/kicad/code/kicad/-/issues/15553
     else
     {
-        solid->SetShape( shape->Clone() );
+        const std::shared_ptr<SHAPE_POLY_SET>& poly = aPad->GetEffectivePolygon( ERROR_OUTSIDE );
+
+        if( poly->OutlineCount() )
+            solid->SetShape( new SHAPE_SIMPLE( poly->Outline( 0 ) ) );
     }
 
     return solid;
@@ -1528,7 +1533,7 @@ void PNS_KICAD_IFACE_BASE::SyncWorld( PNS::NODE *aWorld )
         return;
     }
 
-    int worstClearance = m_board->GetDesignSettings().GetBiggestClearanceValue();
+    int worstClearance = m_board->GetMaxClearanceValue();
 
     m_world = aWorld;
 
