@@ -250,6 +250,8 @@ void BOARD_INSPECTION_TOOL::InspectDRCError( const std::shared_ptr<RC_ITEM>& aDR
                     nullptr, this );
     }
 
+    m_inspectClearanceDialog->DeleteAllPages();
+
     WX_HTML_REPORT_BOX* r = nullptr;
     bool                compileError = false;
     DRC_ENGINE          drcEngine = makeDRCEngine( &compileError );
@@ -503,7 +505,7 @@ void BOARD_INSPECTION_TOOL::InspectDRCError( const std::shared_ptr<RC_ITEM>& aDR
         }
         else
         {
-            constraint = drcEngine.EvalRules( CLEARANCE_CONSTRAINT, a, a, layer, r );
+            constraint = drcEngine.EvalRules( CLEARANCE_CONSTRAINT, a, b, layer, r );
             clearance = constraint.m_Value.Min();
             clearanceStr = m_frame->StringFromValue( clearance, true );
 
@@ -766,7 +768,11 @@ int BOARD_INSPECTION_TOOL::InspectClearance( const TOOL_EVENT& aEvent )
         {
             // Different nets (or one or both unconnected)....
             constraint = drcEngine.EvalRules( CLEARANCE_CONSTRAINT, a, b, layer, r );
-            clearance = constraint.m_Value.Min();
+
+            if( constraint.IsNull() )
+                clearance = 0;
+            else
+                clearance = constraint.m_Value.Min();
 
             if( compileError )
                 reportCompileError( r );
